using backend.Models.NswFuelApi;
using backend.Services;
using Microsoft.AspNetCore.Mvc;

namespace backend.Controllers;

[ApiController]
[Route("api/brands")]
public class BrandController : ControllerBase
{
    private readonly IBrandService _brandService;
    private readonly ILogger<BrandController> _logger;

    public BrandController(IBrandService brandService, ILogger<BrandController> logger)
    {
        _brandService = brandService;
        _logger = logger;
    }

    [HttpGet("all")]
    [EndpointDescription("Retrieves a list of all available brands.")]
    [EndpointSummary("Get all brands")]
    public async Task<ActionResult<IEnumerable<BrandType>>> GetAllBrands()
    {
        _logger.LogInformation("Processing api/brands/all request");
        var brands = await _brandService.GetAllBrands();
        if (!brands.Any()) return NoContent();

        return Ok(brands);
    }

    [HttpGet("state")]
    [EndpointDescription("Retrieves a list of all brands by the state they are located in.")]
    [EndpointSummary("Get brands by state")]
    public async Task<ActionResult<IEnumerable<BrandType>>> GetBrandsByState([FromQuery] string state)
    {
        _logger.LogInformation("Processing api/brands/state request");
        var brands = await _brandService.GetBrandsByState(state);
        if (!brands.Any()) return NotFound($"No brands found in state: {state}");

        return Ok(brands);
    }

    [HttpGet("name")]
    [EndpointDescription(
        "Retrieves a list of all available brands, where the brand name contains the provided search string. Case-insensitive.")]
    [EndpointSummary("Get brands by name")]
    public async Task<ActionResult<IEnumerable<BrandType>>> GetBrandsByName([FromQuery] string name)
    {
        _logger.LogInformation("Processing api/brands/name request");
        var brands = await _brandService.GetBrandsByName(name);
        if (!brands.Any()) return NotFound($"No brands found with name containing: {name}");

        return Ok(brands);
    }
}