using backend.Models.NswFuelApi;
using backend.Models.Utilities;
using backend.Services;
using backend.Validators;
using Microsoft.AspNetCore.Mvc;

namespace backend.Controllers;

[ApiController]
[Route("api/prices")]
public class PriceController : ControllerBase
{
    private readonly IGlobalControllerValidators _globalControllerValidators;
    private readonly ILogger<PriceController> _logger;
    private readonly IPriceService _priceService;

    public PriceController(IPriceService priceService, ILogger<PriceController> logger,
        IGlobalControllerValidators globalControllerValidators)
    {
        _priceService = priceService;
        _logger = logger;
        _globalControllerValidators = globalControllerValidators;
    }

    [HttpGet("current")]
    [EndpointDescription("Retrieves a list of all available prices, with only the most recent displayed.")]
    [EndpointSummary("Get all most recent prices")]
    public async Task<ActionResult<Pagination<Price>>> GetAllCurrentPrices(
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/prices/current request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var prices = await _priceService.GetAllCurrentPrices(pageNumber, pageSize);
        if (!prices.Data.Any()) return NoContent();

        return Ok(prices);
    }

    [HttpGet("current/station")]
    [EndpointDescription("Retrieves a list of all current prices for the station with the code provided.")]
    [EndpointSummary("Get current prices by station code")]
    public async Task<ActionResult<Pagination<Price>>> GetPricesByStationCode(
        [FromQuery] int stationCode,
        [FromQuery] string? fuelType,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/prices/current/station request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var prices = await _priceService.GetCurrentPricesByStationCode(stationCode, fuelType, pageNumber, pageSize);
        if (!prices.Data.Any()) return NotFound($"No prices found for station code {stationCode}");

        return Ok(prices);
    }

    [HttpGet("current/fuel-type")]
    [EndpointDescription("Retrieves a list of all most recent prices for the provided fuel type.")]
    [EndpointSummary("Get all most recent prices by fuel type")]
    public async Task<ActionResult<Pagination<Price>>> GetCurrentPricesByFuelType(
        [FromQuery] string fuelType,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/prices/current/fuel-type request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var prices = await _priceService.GetCurrentPricesByFuelType(fuelType.Trim().ToUpper(), pageNumber, pageSize);
        if (!prices.Data.Any()) return NotFound($"No prices found for fuel type {fuelType.Trim().ToUpper()}");

        return Ok(prices);
    }

    [HttpGet("current/fuel-type/lowest")]
    [EndpointDescription("Retrieves a list of the lowest prices for the provided fuel type.")]
    [EndpointSummary("Get lowest prices by fuel type")]
    public async Task<ActionResult<Pagination<Price>>> GetLowestPricesByFuelType(
        [FromQuery] string fuelType,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/prices/current/fuel-type/lowest request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var prices =
            await _priceService.GetCurrentLowestPricesByFuelType(fuelType.Trim().ToUpper(), pageNumber, pageSize);
        if (!prices.Data.Any()) return NotFound($"No prices found for fuel type {fuelType.Trim().ToUpper()}");

        return Ok(prices);
    }

    [HttpGet("period")]
    [EndpointDescription(
        "Retrieves a list of prices aggregated by a default time bucket of one day. Time values are UTC, in ISO 8601 format: yyyy-MM-ddTHH:mm:ss.SSSZ")]
    [EndpointSummary("Get prices for a specific time period")]
    public async Task<ActionResult<Pagination<Price>>> GetPricesForTimePeriod(
        [FromQuery] DateTimeOffset start,
        [FromQuery] DateTimeOffset end,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/prices/period request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var prices = await _priceService.GetPricesForTimePeriod(start, end, pageNumber, pageSize);
        if (!prices.Data.Any()) return NoContent();

        return Ok(prices);
    }
}