using backend.Models.Trends;
using backend.Services;
using Microsoft.AspNetCore.Mvc;

namespace backend.Controllers;

[ApiController]
[Route("api/trends/daily-averages")]
public class PriceTrendsController : ControllerBase
{
    private readonly ILogger<PriceTrendsController> _logger;
    private readonly IPriceTrendsService _priceTrendsTrendsService;
    private readonly IStationService _stationService;

    public PriceTrendsController(IPriceTrendsService priceTrendsService, ILogger<PriceTrendsController> logger,
        IStationService stationService)
    {
        _priceTrendsTrendsService = priceTrendsService;
        _logger = logger;
        _stationService = stationService;
    }

    [HttpGet("")]
    [EndpointDescription(
        "Retrieves a list of days, and the average price for that day, within the given period, and for the fuel type specified.")]
    [EndpointSummary("Get a list of daily price averages")]
    public async Task<ActionResult<DailyPriceTrends>> GetPricesByStationCode(
        [FromQuery] DateTimeOffset startDate,
        [FromQuery] DateTimeOffset endDate,
        [FromQuery] string fuelType)
    {
        _logger.LogInformation("Processing api/trends/daily-averages request");

        var priceTrends =
            await _priceTrendsTrendsService.GetDailyPriceTrendsForPeriod(startDate, endDate, fuelType.Trim().ToUpper());

        if (priceTrends == null || !priceTrends.DailyAverages.Any()) return NotFound("No price trend data found");

        return Ok(priceTrends);
    }

    [HttpGet("within-radius")]
    [EndpointDescription(
        "Retrieves a list of days, and the average price for that day, within the given period, and for the fuel type and brand specified. Only stations within the given radius are included in the calculations.")]
    [EndpointSummary("Get a list of daily price averages for stations within a given radius")]
    public async Task<ActionResult<DailyPriceTrends>> GetPricesByStationCodeWithinRadius(
        [FromQuery] DateTimeOffset startDate,
        [FromQuery] DateTimeOffset endDate,
        [FromQuery] string fuelType,
        [FromQuery] string? brand,
        [FromQuery] double latitude,
        [FromQuery] double longitude,
        [FromQuery] double radiusInMetres)
    {
        _logger.LogInformation("Processing api/trends/daily-averages/within-radius request");

        var stationCodes =
            (await _stationService.GetStationCodesWithinRadius(latitude, longitude, radiusInMetres, brand)).ToList();
        if (stationCodes.Count == 0) return NotFound("No stations found within that radius");

        var priceTrends = await _priceTrendsTrendsService.GetDailyPriceTrendsForPeriodByStationCodes(
            startDate, endDate, fuelType.Trim().ToUpper(), stationCodes);

        if (priceTrends == null || !priceTrends.DailyAverages.Any()) return NotFound("No price trend data found");

        return Ok(priceTrends);
    }

    [HttpGet("station-codes")]
    [EndpointDescription(
        "Retrieves a list of days, and the price averages for the fuel type on that day across the station codes specified, within the given period.")]
    [EndpointSummary("Get a history of daily prices for the selected stations")]
    public async Task<ActionResult<DailyPriceTrends>> GetDailyPriceAveragesForPeriodByStationCodes(
        [FromQuery] DateTimeOffset startDate,
        [FromQuery] DateTimeOffset endDate,
        [FromQuery] string fuelType,
        [FromQuery] IEnumerable<int> stationCodes)
    {
        _logger.LogInformation("Processing api/trends/daily-history/station-codes request");

        var priceTrends = await _priceTrendsTrendsService.GetDailyPriceTrendsForPeriodByStationCodes(
            startDate, endDate, fuelType.Trim().ToUpper(), stationCodes);

        if (priceTrends == null || !priceTrends.DailyAverages.Any()) return NotFound("No price trend data found");

        return Ok(priceTrends);
    }
}