using backend.Models.NswFuelApi;
using backend.Models.Utilities;
using backend.Services;
using backend.Validators;
using Microsoft.AspNetCore.Mvc;

namespace backend.Controllers;

[ApiController]
[Route("api/stations")]
public class StationController : ControllerBase
{
    private readonly IGlobalControllerValidators _globalControllerValidators;
    private readonly ILogger<StationController> _logger;
    private readonly IStationService _stationService;

    public StationController(IStationService stationService, ILogger<StationController> logger,
        IGlobalControllerValidators globalControllerValidators)
    {
        _stationService = stationService;
        _logger = logger;
        _globalControllerValidators = globalControllerValidators;
    }

    [HttpGet("all")]
    [EndpointDescription("Retrieves a list of all available stations.")]
    [EndpointSummary("Get all stations")]
    public async Task<ActionResult<Pagination<Station>>> GetAllStations(
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/stations/all request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var stations = await _stationService.GetAllStations(pageNumber, pageSize);
        if (!stations.Data.Any()) return NoContent();

        return Ok(stations);
    }

    [HttpGet("station-code")]
    [EndpointDescription("Retrieves a single station, where the station code is equal to the provided station code.")]
    [EndpointSummary("Get a station by station code")]
    public async Task<ActionResult<Station>> GetStationsByStationCode([FromQuery] int stationCode)
    {
        _logger.LogInformation("Processing api/stations/station-code request");
        var station = await _stationService.GetStationByStationCode(stationCode);

        return Ok(station);
    }

    [HttpGet("within-radius")]
    [EndpointDescription(
        "Retrieves a list of all stations within the area described by a circle with centre at the provided coordinates and a radius of the provided value in metres.")]
    [EndpointSummary("Get stations within radius")]
    public async Task<ActionResult<Pagination<Station>>> GetStationsWithinRadius(
        [FromQuery] double latitude,
        [FromQuery] double longitude,
        [FromQuery] double radiusInMetres,
        [FromQuery] string? brand,
        [FromQuery] string? stationName,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/stations/within-radius request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var stations = await _stationService.GetStationsWithinRadius(
            latitude, longitude, radiusInMetres, brand, stationName, pageNumber, pageSize);
        if (!stations.Data.Any()) return NoContent();

        return Ok(stations);
    }
}