using backend.Models.Aggregations;
using backend.Models.Utilities;
using backend.Services;
using backend.Validators;
using Microsoft.AspNetCore.Mvc;

namespace backend.Controllers;

[ApiController]
[Route("api/stations-with-prices")]
public class StationWithPricesController : ControllerBase
{
    private readonly IGlobalControllerValidators _globalControllerValidators;
    private readonly ILogger<StationWithPricesController> _logger;
    private readonly IStationWithPricesService _stationWithPricesService;

    public StationWithPricesController(
        IStationWithPricesService stationWithPricesService,
        ILogger<StationWithPricesController> logger,
        IGlobalControllerValidators globalControllerValidators)
    {
        _stationWithPricesService = stationWithPricesService;
        _logger = logger;
        _globalControllerValidators = globalControllerValidators;
    }

    [HttpGet("station-code")]
    [EndpointDescription(
        "Retrieves a single station along with its current prices, where the station code is equal to the provided station code.")]
    [EndpointSummary("Get a station with prices by station code")]
    public async Task<ActionResult<StationWithPrices>> GetStationWithPricesByStationCode(
        [FromQuery] int stationCode)
    {
        _logger.LogInformation("Processing api/stations-with-prices/station-code request");
        var result = await _stationWithPricesService.GetStationWithPricesByStationCode(stationCode);
        if (result == null)
            return NotFound($"Station with code {stationCode} not found.");

        return Ok(result);
    }

    [HttpGet("within-radius")]
    [EndpointDescription(
        "Retrieves a list of all stations within a specified radius of a given location along with their current prices.")]
    [EndpointSummary("Get stations with prices within radius")]
    public async Task<ActionResult<Pagination<StationWithPrices>>> GetStationsWithPricesWithinRadius(
        [FromQuery] double latitude,
        [FromQuery] double longitude,
        [FromQuery] double radiusInMetres,
        [FromQuery] string? fuelType,
        [FromQuery] string? brand,
        [FromQuery] string? stationName,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/stations-with-prices/within-radius request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var result = await _stationWithPricesService
            .GetStationsWithPricesWithinRadius(
                latitude,
                longitude,
                radiusInMetres,
                fuelType?.Trim().ToUpper(),
                brand,
                stationName,
                pageNumber,
                pageSize
            );

        if (!result.Data.Any())
            return NotFound("No stations found within radius");

        return Ok(result);
    }

    [HttpGet("lowest-prices")]
    [EndpointDescription("Retrieves a list of stations with the lowest prices for a specific fuel type.")]
    [EndpointSummary("Get stations with lowest prices by fuel type")]
    public async Task<ActionResult<Pagination<StationWithPrices>>> GetStationsWithLowestPricesByFuelType(
        [FromQuery] string fuelType,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 10)
    {
        _logger.LogInformation("Processing api/stations-with-prices/lowest-prices request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var result = await _stationWithPricesService
            .GetStationsWithLowestPricesByFuelType(
                fuelType.Trim().ToUpper(),
                pageNumber,
                pageSize
            );

        if (!result.Data.Any())
            return NotFound($"No stations found with fuel type {fuelType.ToUpper()}");

        return Ok(result);
    }

    [HttpGet("lowest-prices/by-fuel-type/within-radius")]
    [EndpointDescription(
        "Retrieves a list of stations with the lowest prices for a specific fuel type, within the specified radius.")]
    [EndpointSummary("Get stations with lowest prices by fuel type, within a radius")]
    public async Task<ActionResult<Pagination<StationWithPrices>>> GetStationsWithLowestPricesByFuelTypeWithinRadius(
        [FromQuery] string fuelType,
        [FromQuery] double latitude,
        [FromQuery] double longitude,
        [FromQuery] double radiusInMetres,
        [FromQuery] int pageNumber = 1,
        [FromQuery] int pageSize = 3)
    {
        _logger.LogInformation("Processing api/stations-with-prices/lowest-prices/by-fuel-type/within-radius request");

        if (!_globalControllerValidators.ValidatePageSize(pageSize))
        {
            _logger.LogError("Page size too large");
            return BadRequest(
                $"Page size too large, maximum page size is {_globalControllerValidators.GetGlobalMaxPageSize()}");
        }

        var result = await _stationWithPricesService
            .GetStationsWithLowestPricesByFuelTypeWithinRadius(
                fuelType.Trim().ToUpper(),
                latitude,
                longitude,
                radiusInMetres,
                pageNumber,
                pageSize
            );

        if (!result.Data.Any())
            return NotFound($"No stations found with fuel type {fuelType.ToUpper()} within the radius specified");

        return Ok(result);
    }
}