using System.Threading.RateLimiting;
using backend.Config;
using backend.Database.Migrations;
using backend.Services;
using backend.Services.Init;
using backend.Validators;
using Hangfire;
using Hangfire.PostgreSql;
using ServiceStack.Data;
using ServiceStack.OrmLite;

namespace backend.Extensions;

public static class ServiceCollectionExtensions
{
    public static IServiceCollection ConfigureServices(this IServiceCollection services, IConfiguration configuration)
    {
        var connectionString = Environment.GetEnvironmentVariable("TIMESCALE_DB_CONNECTION") ??
                               configuration.GetSection("TimescaleDbConfig").Get<TimescaleDbConfig>()!.ConnectionString;

        return services
            .AddSingleton<INswFuelApiService, NswFuelApiService>()
            .AddSingleton<IBrandService, BrandService>()
            .AddSingleton<IPriceService, PriceService>()
            .AddSingleton<IStationService, StationService>()
            .AddSingleton<IFuelTypeService, FuelTypeService>()
            .AddSingleton<IStationWithPricesService, StationWithPricesService>()
            .AddSingleton<IPriceTrendsService, PriceTrendsService>()
            .AddSingleton<ISuburbBoundariesService, SuburbBoundariesService>()
            .AddSingleton<IGlobalControllerValidators, GlobalControllerValidators>()
            .AddSingleton<DatabaseInitialiser>()
            .AddSingleton<HangfireInitialiser>()
            .AddSingleton<MigrationService>()
            .AddLogging(options =>
                options.AddSimpleConsole(c =>
                    c.TimestampFormat = "[dd-MM-yyyy HH:mm:ss] "))
            .AddHttpClient()
            .AddRateLimiter(options =>
            {
                // apply to all endpoints
                options.GlobalLimiter = PartitionedRateLimiter.Create<HttpContext, string>(context =>
                {
                    // grab the first IP in X-Forwarded-For, else fall back
                    var forwarded = context.Request.Headers.TryGetValue("X-Forwarded-For", out var vals)
                        ? vals.FirstOrDefault()?.Split(',')[0]
                        : null;
                    var clientIp = !string.IsNullOrEmpty(forwarded)
                        ? forwarded
                        : context.Connection.RemoteIpAddress?.ToString() ?? "unknown";

                    return RateLimitPartition.GetSlidingWindowLimiter(
                        clientIp,
                        _ => new SlidingWindowRateLimiterOptions
                        {
                            PermitLimit = 100,
                            Window = TimeSpan.FromSeconds(60),
                            SegmentsPerWindow = 60,
                            QueueProcessingOrder = QueueProcessingOrder.OldestFirst,
                            QueueLimit = 0,
                            AutoReplenishment = true
                        }
                    );
                });

                options.OnRejected = async (context, ct) =>
                {
                    var logger = context.HttpContext.RequestServices
                        .GetRequiredService<ILoggerFactory>()
                        .CreateLogger("RateLimiter");

                    var ip = context.HttpContext.Request.Headers["X-Forwarded-For"]
                                 .FirstOrDefault()?.Split(',')[0]
                             ?? context.HttpContext.Connection.RemoteIpAddress?.ToString();

                    logger.LogWarning("Client rate limit exceeded for {ClientIp}", ip);

                    context.HttpContext.Response.StatusCode = StatusCodes.Status429TooManyRequests;
                    await context.HttpContext.Response
                        .WriteAsync("Rate limit exceeded, please try again in one minute", ct);
                };
            })
            .AddHangfire(config =>
                config.UsePostgreSqlStorage(c =>
                    c.UseNpgsqlConnection(connectionString)))
            .AddHangfireServer();
    }

    public static void SetupConfiguration(this IServiceCollection services, IConfiguration configuration)
    {
        var nswFuelApiConfig = new NswFuelApiConfig
        {
            BaseUrl = Environment.GetEnvironmentVariable("NSW_FUEL_API_BASE_URL") ??
                      configuration.GetSection(nameof(NswFuelApiConfig)).Get<NswFuelApiConfig>()?.BaseUrl!,
            ApiKey = Environment.GetEnvironmentVariable("NSW_FUEL_API_KEY") ??
                     configuration.GetSection(nameof(NswFuelApiConfig)).Get<NswFuelApiConfig>()?.ApiKey!,
            ApiSecret = Environment.GetEnvironmentVariable("NSW_FUEL_API_SECRET") ??
                        configuration.GetSection(nameof(NswFuelApiConfig)).Get<NswFuelApiConfig>()?.ApiSecret!,
            AuthorisationHeader = Environment.GetEnvironmentVariable("NSW_FUEL_API_AUTH_HEADER") ??
                                  configuration.GetSection(nameof(NswFuelApiConfig)).Get<NswFuelApiConfig>()
                                      ?.AuthorisationHeader!
        };

        services
            .AddSingleton(nswFuelApiConfig)
            .AddSingleton(configuration.GetSection(nameof(SuburbBoundaryApiConfig)).Get<SuburbBoundaryApiConfig>()!);
    }

    public static void SetupDatabase(this IServiceCollection services, IConfiguration configuration)
    {
        var connectionString = Environment.GetEnvironmentVariable("TIMESCALE_DB_CONNECTION") ??
                               configuration.GetSection("TimescaleDbConfig").Get<TimescaleDbConfig>()!.ConnectionString;

        services.AddSingleton<IDbConnectionFactory>(_ =>
            new OrmLiteConnectionFactory(connectionString, PostgreSqlDialect.Provider));
    }
}