using backend.Database.Migrations;
using backend.Models.NswFuelApi;
using backend.Models.SuburbBoundariesApi;
using ServiceStack.Data;
using ServiceStack.OrmLite;

namespace backend.Services.Init;

public class DatabaseInitialiser
{
    private readonly IDbConnectionFactory _dbFactory;
    private readonly ILogger<DatabaseInitialiser> _logger;
    private readonly MigrationService _migrationService;

    public DatabaseInitialiser(IDbConnectionFactory dbFactory, ILogger<DatabaseInitialiser> logger,
        MigrationService migrationService)
    {
        _dbFactory = dbFactory;
        _logger = logger;
        _migrationService = migrationService;
    }

    public void Initialise()
    {
        _logger.LogInformation("Beginning database initialisation");
        using var db = _dbFactory.Open();

        //create the standard Postgres tables for basic data
        db.CreateTableIfNotExists<BrandType>();
        db.CreateTableIfNotExists<FuelType>();
        db.CreateTableIfNotExists<Station>();
        db.CreateTableIfNotExists<SuburbBoundary>();

        //check if the prices hypertable already exists, otherwise the raw SQL will fail
        if (!db.TableExists<Price>())
        {
            //create the Postgres table for prices, this will make it work correctly with OrmLite as well
            db.CreateTable<Price>();
            //now we need to convert the Price table into a hypertable for time-series use in Timescale
            //time interval is set to 1 day here, but with more memory available to the database, this
            //could easily be expanded to 1 week, perhaps even 1 month
            db.ExecuteSql("SELECT create_hypertable('price', by_range('created_at', INTERVAL '1 day'))");
        }

        _logger.LogInformation("Database schema has been initialised");

        //check if PostGIS is installed, and add it if not
        var postGisInstalled = db.SqlScalar<int>("SELECT COUNT(*) FROM pg_extension WHERE extname = 'postgis';");
        if (postGisInstalled == 0)
        {
            _logger.LogInformation("Attempting to install postgis");
            db.ExecuteSql("CREATE EXTENSION postgis;");
        }

        _logger.LogInformation("Database initialisation complete");

        _logger.LogInformation("Starting database migrations");
        _migrationService.RunMigration1001();
        _logger.LogInformation("Database migrations complete");
    }
}