using System.Net.Http.Headers;
using System.Text.Json;
using backend.Config;
using backend.Models.NswFuelApi;
using ServiceStack.Data;
using ServiceStack.OrmLite;

namespace backend.Services;

public class NswFuelApiService : INswFuelApiService
{
    private readonly IDbConnectionFactory _dbConnectionFactory;
    private readonly IHttpClientFactory _httpClientFactory;
    private readonly ILogger<NswFuelApiService> _logger;
    private readonly NswFuelApiConfig _nswFuelApiConfig;
    private AccessTokenResponse? _cachedToken;

    public NswFuelApiService(IHttpClientFactory httpClientFactory, ILogger<NswFuelApiService> logger,
        NswFuelApiConfig nswFuelApiConfig, IDbConnectionFactory dbConnectionFactory)
    {
        _httpClientFactory = httpClientFactory;
        _nswFuelApiConfig = nswFuelApiConfig;
        _logger = logger;
        _dbConnectionFactory = dbConnectionFactory;
    }

    public async Task<bool> GetLovsAsync()
    {
        var client = _httpClientFactory.CreateClient();
        var token = await GetAccessTokenAsync();
        var url = $"{_nswFuelApiConfig.BaseUrl}/FuelCheckRefData/v2/fuel/lovs";
        var request = CreateRequestMessage(HttpMethod.Get, url,
            token?.AccessToken);

        var response = await client.SendAsync(request);
        response.EnsureSuccessStatusCode();

        var content = await response.Content.ReadAsStreamAsync();
        var referenceData = await JsonSerializer.DeserializeAsync<ReferenceData>(content);
        if (referenceData == null)
        {
            _logger.LogError("No reference data returned from onegov API");
            return false;
        }

        using var db = _dbConnectionFactory.OpenDbConnection();
        var transaction = db.OpenTransaction();

        try
        {
            var existingBrands = await db.SelectAsync<BrandType>();
            var newBrandsToAdd = referenceData.Brands.Items.Where(b =>
                existingBrands.All(eb =>
                    eb.Name != b?.Name));
            db.BulkInsert(newBrandsToAdd);

            var existingFuelTypes = await db.SelectAsync<FuelType>();
            var newFuelTypesToAdd = referenceData.FuelTypes.Items.Where(ft =>
                existingFuelTypes.All(eft =>
                    eft.Code != ft?.Code));
            db.BulkInsert(newFuelTypesToAdd);

            var existingStations = await db.SelectAsync<Station>();

            var mappedStationData = referenceData.Stations.Items.Select(dto => new Station
            {
                State = dto!.State,
                Code = int.Parse(dto.Code),
                Address = dto.Address,
                Brand = dto.Brand,
                Name = dto.Name,
                Geography = $"SRID=7844;POINT({dto.Location.Longitude} {dto.Location.Latitude})"
            });

            //stations which exist in the DB, but not in the reference data (they have been closed or updated)
            var removedStations =
                existingStations.Where(es => mappedStationData.All(ms => ms.Code != es.Code)).ToList();
            foreach (var rs in removedStations) rs.DeletedAt = DateTimeOffset.UtcNow;

            await db.UpdateAllAsync(removedStations);

            //stations which do not exist in the DB, but exist in the reference data (they are new)
            var newStationsToAdd = mappedStationData.Where(s =>
                existingStations.All(es =>
                    es.Code != s.Code));

            db.BulkInsert(newStationsToAdd);
            transaction.Commit();
        }
        catch (Exception e)
        {
            _logger.LogError("Error while attempting to get lovs: {Error}", e);
            transaction.Rollback();
            return false;
        }

        _logger.LogInformation("Successfully updated reference data");
        return true;
    }

    public async Task<bool> GetCurrentPricesAsync()
    {
        var client = _httpClientFactory.CreateClient();
        var token = await GetAccessTokenAsync();
        var url = $"{_nswFuelApiConfig.BaseUrl}/FuelPriceCheck/v2/fuel/prices";
        var request = CreateRequestMessage(HttpMethod.Get, url, token?.AccessToken);

        var response = await client.SendAsync(request);
        response.EnsureSuccessStatusCode();

        var content = await response.Content.ReadAsStreamAsync();
        var locationPrices = await JsonSerializer.DeserializeAsync<LocationPrices>(content);
        if (locationPrices == null)
        {
            _logger.LogError("No price data returned from onegov API");
            return false;
        }

        var mappedPrices = locationPrices.PriceDtos.Select(dto => new Price
        {
            StationCode = dto.StationCode,
            FuelType = dto.FuelType.Trim().ToUpper(), //ensure that fuel type is always upper
            PriceValue = dto.PriceValue,
            LastUpdated = DateTimeOffset.ParseExact(dto.LastUpdated, "dd/MM/yyyy HH:mm:ss", null).ToUniversalTime(),
            State = dto.State
        }).ToList();

        using var db = _dbConnectionFactory.OpenDbConnection();
        var transaction = db.OpenTransaction();

        try
        {
            //stage 1: retrieve the latest price for each StationCode and FuelType combination
            var latestPricesQuery = db.From<Price>()
                .Where(p => mappedPrices.Select(mp => mp.StationCode).Contains(p.StationCode)
                            && mappedPrices.Select(mp => mp.FuelType).Contains(p.FuelType))
                .Select(p => new { p.StationCode, p.FuelType, p.LastUpdated });

            var latestPrices = db.Select(latestPricesQuery)
                .GroupBy(p => new { p.StationCode, p.FuelType })
                .Select(g => new
                {
                    g.Key.StationCode,
                    g.Key.FuelType,
                    LastUpdated = g.Max(x => x.LastUpdated)
                })
                .ToList();

            //stage 2: filter out prices that are not new or updated
            var pricesToAdd = mappedPrices.Where(newPrice =>
            {
                var latestPrice = latestPrices.FirstOrDefault(lp =>
                    lp.StationCode == newPrice.StationCode && lp.FuelType == newPrice.FuelType);

                return latestPrice == null || newPrice.LastUpdated > latestPrice.LastUpdated;
            }).ToList();

            //stage 3: insert only new or updated prices
            if (pricesToAdd.Count > 0)
            {
                _logger.LogInformation("Attempting to insert {Count} new prices", pricesToAdd.Count);
                db.BulkInsert(pricesToAdd);
            }
            else
            {
                _logger.LogInformation("No new prices to insert");
            }

            transaction.Commit();
        }
        catch (Exception e)
        {
            _logger.LogError("Error while attempting to get current prices: {Error}", e);
            transaction.Rollback();
            return false;
        }

        _logger.LogInformation("Successfully updated current prices");
        return true;
    }

    private async Task<AccessTokenResponse?> GetAccessTokenAsync()
    {
        if (_cachedToken != null)
        {
            //parse the issued_at and expires_in strings to long
            var issuedAtUnixTime = long.Parse(_cachedToken.IssuedAt);
            var expiresInSeconds = long.Parse(_cachedToken.ExpiresIn);

            //convert issued_at from UNIX timestamp to DateTime
            var issuedAtDateTime = DateTimeOffset.FromUnixTimeMilliseconds(issuedAtUnixTime).DateTime;

            //calculate expiration time
            var expirationTime = issuedAtDateTime.AddSeconds(expiresInSeconds);

            //check if the token is expired or about to expire within the next 5 minutes (300 seconds)
            if (expirationTime > DateTime.Now.AddSeconds(300)) return _cachedToken;
        }

        _logger.LogInformation("Cached token has expired, getting a new one");

        var client = _httpClientFactory.CreateClient();
        var url = $"{_nswFuelApiConfig.BaseUrl}/oauth/client_credential/accesstoken?grant_type=client_credentials";
        var request = new HttpRequestMessage(HttpMethod.Get,
            url);
        request.Headers.Authorization = new AuthenticationHeaderValue("Basic", _nswFuelApiConfig.AuthorisationHeader);

        var response = await client.SendAsync(request);
        response.EnsureSuccessStatusCode();
        _logger.LogInformation("Successfully retrieved a new access token");

        var content = await response.Content.ReadAsStreamAsync();
        var tokenResponse = await JsonSerializer.DeserializeAsync<AccessTokenResponse>(content);
        _cachedToken = tokenResponse;
        return tokenResponse;
    }

    #region Helpers

    private HttpRequestMessage CreateRequestMessage(HttpMethod method, string url, string? accessToken)
    {
        var request = new HttpRequestMessage(method, url);

        if (!string.IsNullOrEmpty(accessToken))
            request.Headers.Authorization = new AuthenticationHeaderValue("Bearer", accessToken);

        request.Headers.Add("apikey", _nswFuelApiConfig.ApiKey);
        request.Headers.Add("transactionid", Guid.NewGuid().ToString());
        request.Headers.Add("requesttimestamp", DateTime.UtcNow.ToString("R"));
        request.Headers.Add("if-modified-since", DateTime.UtcNow.ToString("R"));

        return request;
    }

    #endregion
}