using backend.Models.NswFuelApi;
using backend.Models.Utilities;
using ServiceStack.Data;
using ServiceStack.OrmLite;
using ServiceStack.OrmLite.PostgreSQL;

namespace backend.Services;

public class PriceService : IPriceService
{
    private readonly IDbConnectionFactory _dbConnectionFactory;

    public PriceService(IDbConnectionFactory dbConnectionFactory)
    {
        _dbConnectionFactory = dbConnectionFactory;
    }

    public async Task<Pagination<Price>> GetAllCurrentPrices(int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        const string sql = """
                               SELECT DISTINCT ON (station_code, fuel_type)
                                   id, station_code, fuel_type, price_value, last_updated, state, created_at
                               FROM
                                   price
                               ORDER BY
                                   station_code, fuel_type, created_at DESC
                               LIMIT @PageSize OFFSET @Offset;
                           """;

        const string countSql = """
                                    SELECT COUNT(*) FROM (
                                        SELECT DISTINCT ON (station_code, fuel_type)
                                            id, station_code, fuel_type, price_value, last_updated, state, created_at
                                        FROM
                                            price
                                    ) AS distinct_prices;
                                """;
        var totalCount = await db.SqlScalarAsync<int>(countSql);

        var prices =
            await db.SqlListAsync<Price>(sql, new { Offset = (pageNumber - 1) * pageSize, PageSize = pageSize });

        return new Pagination<Price>(prices, totalCount, pageNumber, pageSize);
    }

    public async Task<Pagination<Price>> GetCurrentPricesByStationCode(int stationCode, string? fuelType,
        int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        const string sql = """
                               SELECT DISTINCT ON (station_code, fuel_type)
                                   id, station_code, fuel_type, price_value, last_updated, state, created_at
                               FROM
                                   price
                               WHERE
                                   station_code = @StationCode
                               AND (@FuelType IS NULL OR fuel_type = @FuelType)
                               ORDER BY
                                   station_code, fuel_type, created_at DESC
                               LIMIT @PageSize OFFSET @Offset;
                           """;

        const string countSql = """
                                    SELECT COUNT(*) FROM (
                                        SELECT DISTINCT ON (station_code, fuel_type)
                                            id, station_code, fuel_type, price_value, last_updated, state, created_at
                                        FROM
                                            price
                                        WHERE
                                            station_code = @StationCode
                                        AND (@FuelType IS NULL OR fuel_type = @FuelType)
                                    ) AS distinct_prices;
                                """;
        var totalCount = await db.SqlScalarAsync<int>(countSql, new { StationCode = stationCode, FuelType = fuelType });

        var prices = await db.SqlListAsync<Price>(sql,
            new
            {
                StationCode = stationCode, FuelType = fuelType, Offset = (pageNumber - 1) * pageSize,
                PageSize = pageSize
            });

        return new Pagination<Price>(prices, totalCount, pageNumber, pageSize);
    }

    public async Task<Pagination<Price>> GetCurrentPricesByStationCodes(IEnumerable<int> stationCodes, string? fuelType,
        int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        var stationCodesArray = stationCodes.ToArray();
        var sql = $"""
                       SELECT DISTINCT ON (station_code, fuel_type)
                           id, station_code, fuel_type, price_value, last_updated, state, created_at
                       FROM
                           price
                       WHERE
                           station_code = ANY({PgSql.Array(stationCodesArray)})
                       AND (@FuelType IS NULL OR fuel_type = @FuelType)
                       ORDER BY
                           station_code, fuel_type, created_at DESC
                       LIMIT @PageSize OFFSET @Offset;
                   """;

        var countSql = $"""
                            SELECT COUNT(*) FROM (
                                SELECT DISTINCT ON (station_code, fuel_type)
                                    id, station_code, fuel_type, price_value, last_updated, state, created_at
                                FROM
                                    price
                                WHERE
                                    station_code = ANY({PgSql.Array(stationCodesArray)})
                                AND (@FuelType IS NULL OR fuel_type = @FuelType)
                            ) AS distinct_prices;
                        """;

        var totalCount = await db.SqlScalarAsync<int>(countSql, new { FuelType = fuelType });

        var prices = await db.SqlListAsync<Price>(sql,
            new
            {
                FuelType = fuelType,
                Offset = (pageNumber - 1) * pageSize,
                PageSize = pageSize
            });

        return new Pagination<Price>(prices, totalCount, pageNumber, pageSize);
    }

    public async Task<Pagination<Price>> GetCurrentPricesByFuelType(string fuelType, int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        const string sql = """
                               SELECT DISTINCT ON (station_code, fuel_type)
                                   id, station_code, fuel_type, price_value, last_updated, state, created_at
                               FROM
                                   price
                               WHERE
                                   fuel_type = @FuelType
                               ORDER BY
                                   station_code, fuel_type, created_at DESC
                               LIMIT @PageSize OFFSET @Offset;
                           """;

        const string countSql = """
                                    SELECT COUNT(*) FROM (
                                        SELECT DISTINCT ON (station_code, fuel_type)
                                            id, station_code, fuel_type, price_value, last_updated, state, created_at
                                        FROM
                                            price
                                        WHERE
                                            fuel_type = @FuelType
                                    ) AS distinct_prices;
                                """;
        var totalCount = await db.SqlScalarAsync<int>(countSql, new { FuelType = fuelType });

        var prices = await db.SqlListAsync<Price>(sql,
            new { FuelType = fuelType, Offset = (pageNumber - 1) * pageSize, PageSize = pageSize });

        return new Pagination<Price>(prices, totalCount, pageNumber, pageSize);
    }

    public async Task<Pagination<Price>> GetCurrentLowestPricesByFuelType(string fuelType, int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        //stage 1: get all prices by fuel type
        var prices = await db.SelectAsync<Price>(p => p.FuelType == fuelType);

        //stage 2: group by station code and select the lowest price
        var lowestPrices = prices
            .GroupBy(p => p.StationCode)
            .Select(g => g.OrderBy(p => p.PriceValue).ThenByDescending(p => p.CreatedAt).First())
            .OrderBy(p => p.PriceValue)
            .Skip((pageNumber - 1) * pageSize)
            .Take(pageSize)
            .ToList();

        //stage 3: count the distinct stations for pagination
        var totalCount = prices.Select(p => p.StationCode).Distinct().Count();

        return new Pagination<Price>(lowestPrices, totalCount, pageNumber, pageSize);
    }

    public async Task<Pagination<Price>> GetPricesForTimePeriod(DateTimeOffset start, DateTimeOffset end,
        int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        var totalCount = await db.CountAsync<Price>(p => p.CreatedAt >= start && p.CreatedAt <= end);

        var prices = await db.SelectAsync(db.From<Price>()
            .Where(p => p.CreatedAt >= start && p.CreatedAt <= end)
            .OrderBy(p => p.StationCode)
            .Skip((pageNumber - 1) * pageSize)
            .Take(pageSize));

        return new Pagination<Price>(prices, totalCount, pageNumber, pageSize);
    }
}