using backend.Models.Trends;
using ServiceStack.Data;
using ServiceStack.OrmLite;
using ServiceStack.OrmLite.PostgreSQL;

namespace backend.Services;

public class PriceTrendsService : IPriceTrendsService
{
    private readonly IDbConnectionFactory _dbConnectionFactory;
    private readonly ILogger<PriceTrendsService> _logger;

    public PriceTrendsService(IDbConnectionFactory dbConnectionFactory, ILogger<PriceTrendsService> logger)
    {
        _dbConnectionFactory = dbConnectionFactory;
        _logger = logger;
    }

    public async Task<DailyPriceTrends?> GetDailyPriceTrendsForPeriod(
        DateTimeOffset startDate, DateTimeOffset endDate, string fuelType)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        //query to get the daily averages
        const string dailyAveragesSql = """
                                            SELECT
                                                time_bucket('1 day', created_at) AS Date,
                                                AVG(price_value) AS AveragePriceValue
                                            FROM price
                                            WHERE created_at BETWEEN @StartDate AND @EndDate
                                            AND fuel_type = @FuelType
                                            GROUP BY Date
                                            ORDER BY Date;
                                        """;

        //query to get the overall max, min, and period average values
        const string statisticsSql = """
                                       SELECT
                                           MAX(price_value) AS MaxPriceValue,
                                           MIN(price_value) AS MinPriceValue,
                                           AVG(price_value) AS PeriodAverage
                                       FROM price
                                       WHERE created_at BETWEEN @StartDate AND @EndDate
                                       AND fuel_type = @FuelType;
                                     """;

        var dailyAverages = await db.SelectAsync<DailyPriceAverage>(dailyAveragesSql,
            new { StartDate = startDate, EndDate = endDate, FuelType = fuelType });

        if (dailyAverages.Count == 0)
        {
            _logger.LogError("No daily averages were calculated for the price trends calculation");
            return null;
        }

        var statistics = await db.SingleAsync<DailyPriceTrendsStatistics>(statisticsSql,
            new { StartDate = startDate, EndDate = endDate, FuelType = fuelType });

        if (statistics == null)
        {
            _logger.LogError("No statistics were calculated for the price trends calculation");
            return null;
        }

        var dailyMax = dailyAverages.Select(x => x.AveragePriceValue).Max();
        var dailyMin = dailyAverages.Select(x => x.AveragePriceValue).Min();

        statistics.DailyAverageMax = dailyMax;
        statistics.DailyAverageMin = dailyMin;

        var trends = new DailyPriceTrends
        {
            DailyAverages = dailyAverages,
            Statistics = statistics
        };

        return trends;
    }

    public async Task<DailyPriceTrends?> GetDailyPriceTrendsForPeriodByStationCodes(DateTimeOffset startDate,
        DateTimeOffset endDate, string fuelType,
        IEnumerable<int> stationCodes)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        var stationCodesArray = stationCodes.ToArray();
        //query to get the daily averages
        var dailyAveragesSql = $"""
                                    SELECT
                                        time_bucket('1 day', created_at) AS Date,
                                        AVG(price_value) AS AveragePriceValue
                                    FROM price
                                    WHERE created_at BETWEEN @StartDate AND @EndDate
                                    AND fuel_type = @FuelType
                                    AND  station_code = ANY({PgSql.Array(stationCodesArray)})
                                    GROUP BY Date
                                    ORDER BY Date;
                                """;

        //query to get the overall max, min, and period average values
        var statisticsSql = $"""
                               SELECT
                                   MAX(price_value) AS MaxPriceValue,
                                   MIN(price_value) AS MinPriceValue,
                                   AVG(price_value) AS PeriodAverage
                               FROM price
                               WHERE created_at BETWEEN @StartDate AND @EndDate
                               AND fuel_type = @FuelType
                               AND station_code = ANY({PgSql.Array(stationCodesArray)});
                             """;

        var dailyAverages = await db.SelectAsync<DailyPriceAverage>(dailyAveragesSql,
            new { StartDate = startDate, EndDate = endDate, FuelType = fuelType });

        if (dailyAverages.Count == 0)
        {
            _logger.LogError("No daily averages were found");
            return null;
        }

        var statistics = await db.SingleAsync<DailyPriceTrendsStatistics>(statisticsSql,
            new { StartDate = startDate, EndDate = endDate, FuelType = fuelType });

        if (statistics == null)
        {
            _logger.LogError("No statistics were found");
            return null;
        }

        var dailyMax = dailyAverages.Select(x => x.AveragePriceValue).Max();
        var dailyMin = dailyAverages.Select(x => x.AveragePriceValue).Min();

        statistics.DailyAverageMax = dailyMax;
        statistics.DailyAverageMin = dailyMin;

        var trends = new DailyPriceTrends
        {
            DailyAverages = dailyAverages,
            Statistics = statistics
        };

        return trends;
    }
}