using backend.Models.NswFuelApi;
using backend.Models.Utilities;
using ServiceStack.Data;
using ServiceStack.OrmLite;

namespace backend.Services;

public class StationService : IStationService
{
    private readonly IDbConnectionFactory _dbConnectionFactory;

    public StationService(IDbConnectionFactory dbConnectionFactory)
    {
        _dbConnectionFactory = dbConnectionFactory;
    }

    public async Task<Pagination<Station>> GetAllStations(int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        var totalCount = await db.CountAsync<Station>();

        var stations = await db.SelectAsync(db.From<Station>()
            //do not show soft-deleted stations
            .Where(x => x.DeletedAt == null)
            .OrderBy(x => x.Code)
            .Limit((pageNumber - 1) * pageSize, pageSize));

        return new Pagination<Station>(stations, totalCount, pageNumber, pageSize);
    }

    public async Task<Station> GetStationByStationCode(int stationCode)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();
        var station = await db.SingleAsync<Station>(x => x.Code == stationCode && x.DeletedAt == null);
        return station;
    }

    public async Task<Pagination<Station>> GetStationsWithinRadius(double latitude, double longitude,
        double radiusInMetres, string? brand, string? stationName, int pageNumber, int pageSize)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        //sql to use PostGIS to find all stations within the radius from the point
        const string sql = """
                           SELECT *,
                               ST_Distance(
                                   geography::geography,
                                   ST_SetSRID(ST_MakePoint(@Longitude, @Latitude), 7844)::geography
                               ) AS distance
                           FROM station
                           WHERE ST_DWithin(
                               geography::geography,
                               ST_SetSRID(ST_MakePoint(@Longitude, @Latitude), 7844)::geography,
                               @Radius
                           ) 
                           AND (@Brand IS NULL OR brand = @Brand)
                           AND (@StationName IS NULL OR LOWER(name) ILIKE '%' || LOWER(@StationName) || '%')
                           AND (deleted_at IS NULL)
                           ORDER BY distance
                           LIMIT @PageSize OFFSET @Offset
                           """;

        var offset = (pageNumber - 1) * pageSize;

        //use station with distance to map the distance values
        var stationWithDistances = await db.SqlListAsync<StationWithDistance>(sql,
            new
            {
                Latitude = latitude,
                Longitude = longitude,
                Radius = radiusInMetres,
                Brand = brand,
                StationName = stationName,
                PageSize = pageSize,
                Offset = offset
            });

        // Convert DTOs to Station objects
        var stations = stationWithDistances.Select(dto => new Station
        {
            Id = dto.Id,
            Brand = dto.Brand,
            Code = dto.Code,
            Name = dto.Name,
            Address = dto.Address,
            Geography = dto.Geography,
            State = dto.State,
            CreatedAt = dto.CreatedAt,
            Distance = dto.Distance // Map the distance
        }).ToList();

        const string countSql = """
                                SELECT COUNT(*) 
                                FROM station 
                                WHERE ST_DWithin(
                                    geography::geography,
                                    ST_SetSRID(ST_MakePoint(@Longitude, @Latitude), 7844)::geography,
                                    @Radius
                                )
                                AND (@Brand IS NULL OR brand = @Brand)
                                AND (@StationName IS NULL OR LOWER(name) ILIKE '%' || LOWER(@StationName) || '%')
                                AND (deleted_at IS NULL)
                                """;

        var totalCount = await db.ScalarAsync<long>(countSql,
            new
            {
                Latitude = latitude, Longitude = longitude, Radius = radiusInMetres, Brand = brand,
                StationName = stationName
            });

        return new Pagination<Station>(stations, totalCount, pageNumber, pageSize);
    }

    public async Task<IEnumerable<int>> GetStationCodesWithinRadius(double latitude, double longitude,
        double radiusInMetres, string? brand)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        //sql to use PostGIS to find all station codes within the radius from the point
        const string sql = """
                           SELECT code FROM station
                           WHERE ST_DWithin(
                               geography::geography,
                               ST_SetSRID(ST_MakePoint(@Longitude, @Latitude), 7844)::geography,
                               @Radius
                           ) 
                           AND (@Brand IS NULL OR brand = @Brand)
                           AND (deleted_at IS NULL)
                           """;

        //use station with distance to map the distance values
        var stationCodes = await db.SqlListAsync<int>(sql,
            new
            {
                Latitude = latitude,
                Longitude = longitude,
                Radius = radiusInMetres,
                Brand = brand
            });

        return stationCodes;
    }
}