using backend.Models.Aggregations;
using backend.Models.NswFuelApi;
using backend.Models.Utilities;
using ServiceStack.Data;
using ServiceStack.OrmLite;

namespace backend.Services;

public class StationWithPricesService : IStationWithPricesService
{
    private readonly IDbConnectionFactory _dbConnectionFactory;
    private readonly IPriceService _priceService;
    private readonly IStationService _stationService;

    public StationWithPricesService(IStationService stationService, IPriceService priceService,
        IDbConnectionFactory dbConnectionFactory)
    {
        _stationService = stationService;
        _priceService = priceService;
        _dbConnectionFactory = dbConnectionFactory;
    }

    public async Task<StationWithPrices?> GetStationWithPricesByStationCode(int stationCode)
    {
        using var db = _dbConnectionFactory.OpenDbConnection();

        var q = db.From<Station>()
            .LeftJoin<Price>((station, price) => station.Code == price.StationCode)
            .Where<Station>(s => s.Code == stationCode && s.DeletedAt == null);

        var results = await db.SelectMultiAsync<Station, Price>(q.SelectDistinct());

        if (results == null || results.Count == 0)
            return null;

        var station = results.First().Item1;

        var prices = results.Select(x => x.Item2).Where(p => p != null)
            .DistinctBy(p => p.FuelType).OrderByDescending(p => p.CreatedAt);

        return new StationWithPrices
        {
            Station = station,
            Prices = prices
        };
    }

    public async Task<Pagination<StationWithPrices>> GetStationsWithPricesWithinRadius(
        double latitude,
        double longitude,
        double radiusInMetres,
        string? fuelType,
        string? brand,
        string? stationName,
        int pageNumber,
        int pageSize)
    {
        //fetch all stations within the radius
        var allStations =
            await _stationService.GetStationsWithinRadius(latitude, longitude, radiusInMetres, brand, stationName, 1,
                int.MaxValue);

        //pull out the station codes from the result
        var stationCodes = allStations.Data.Select(s => s.Code);

        //fetch the prices for the list of station codes
        var prices = await _priceService.GetCurrentPricesByStationCodes(stationCodes, fuelType, 1, int.MaxValue);

        //there should be fewer prices than stations, since not all stations in the radius will have the selected fuel type
        //iterate over the prices (less to loop over) and find the matching station, projecting into a StationWithPrices
        List<StationWithPrices> stationsWithPrices = [];
        stationsWithPrices.AddRange(prices.Data.Select(p => new StationWithPrices
            { Station = allStations.Data.First(s => s.Code == p.StationCode), Prices = [p] }));

        //perform manual pagination on the stations with prices, and order by price value
        var totalFilteredStations = stationsWithPrices.Count;
        var paginatedStations = stationsWithPrices
            .OrderBy(swp => swp.Prices.First().PriceValue)
            .Skip((pageNumber - 1) * pageSize)
            .Take(pageSize)
            .ToList();

        //return paginated results
        return new Pagination<StationWithPrices>(
            paginatedStations,
            totalFilteredStations,
            pageNumber,
            pageSize);
    }

    public async Task<Pagination<StationWithPrices>> GetStationsWithLowestPricesByFuelType(string fuelType,
        int pageNumber, int pageSize)
    {
        var lowestPrices = await _priceService.GetCurrentLowestPricesByFuelType(fuelType, pageNumber, pageSize);
        var stationsWithPrices = new List<StationWithPrices>();

        foreach (var price in lowestPrices.Data)
        {
            var station = await _stationService.GetStationByStationCode(price.StationCode);
            stationsWithPrices.Add(new StationWithPrices
            {
                Station = station,
                Prices = new List<Price> { price }
            });
        }

        return new Pagination<StationWithPrices>(stationsWithPrices, lowestPrices.TotalCount, pageNumber, pageSize);
    }

    public async Task<Pagination<StationWithPrices>> GetStationsWithLowestPricesByFuelTypeWithinRadius(string fuelType,
        double latitude, double longitude, double radiusInMetres, int pageNumber, int pageSize)
    {
        var stations =
            await _stationService.GetStationsWithinRadius(latitude, longitude, radiusInMetres, null, null, pageNumber,
                pageSize);
        var stationWithPricesList = new List<StationWithPrices>();

        foreach (var station in stations.Data)
        {
            var prices = await _priceService.GetCurrentPricesByStationCode(station.Code, fuelType, 1, int.MaxValue);
            var priceForFuelType = prices.Data.FirstOrDefault();
            if (priceForFuelType == null) continue;

            stationWithPricesList.Add(new StationWithPrices
            {
                Station = station,
                Prices = new List<Price> { priceForFuelType }
            });
        }

        return new Pagination<StationWithPrices>(stationWithPricesList, stations.TotalCount, pageNumber, pageSize);
    }
}