"use client";

import { LineChart } from "@mantine/charts";
import {
  Button,
  Container,
  Grid,
  Group,
  NumberInput,
  Select,
  Skeleton,
  Stack,
  Title,
} from "@mantine/core";
import { notifications } from "@mantine/notifications";
import { useQuery } from "@tanstack/react-query";
import axios from "axios";
import dynamic from "next/dynamic";
import { useMemo, useState } from "react";
import { FuelTypeApi, PriceTrendsApi } from "../../generated";
import { Coordinate } from "../../models/Coordinate";
import { DateTimeRange } from "../../models/DateTimeRange";
import { TimePeriod } from "../../models/TimePeriod";
import { getDateRangeForPeriod } from "../../utils/dateRangePickerUtils";
import { DateToDayMonthYearAbbrev } from "../../utils/dateStringUtils";
import {
  GetDefaultFuelType,
  GetHomeLocation,
} from "../../utils/userPreferencesUtils";
import DateRangePicker from "../components/dates/DateRangePicker";
const CustomMap = dynamic(() => import("../components/map/CustomMap"), {
  ssr: false,
});

export default function ChartsPage() {
  const homeLocation = GetHomeLocation();
  const fuelTypeApi = new FuelTypeApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const priceTrendsApi = new PriceTrendsApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );

  const [selectedFuelType, setSelectedFuelType] = useState(
    GetDefaultFuelType()
  );
  const [selectedDateTimeRange, setSelectedDateTimeRange] =
    useState<DateTimeRange>(getDateRangeForPeriod(TimePeriod.Last30Days));
  const [selectedRadius, setSelectedRadius] = useState(10);
  const [selectedPosition, setSelectedPosition] = useState<
    Coordinate | undefined
  >(homeLocation);

  async function FetchFuelTypes() {
    try {
      const fuelTypeResponse = await fuelTypeApi.apiFuelTypesAllGet();
      return fuelTypeResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching the fuel type data. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  async function FetchPriceTrendsAllStations() {
    try {
      const priceTrendsResponse =
        await priceTrendsApi.apiTrendsDailyAveragesGet({
          startDate: selectedDateTimeRange?.startDate?.toISOString(),
          endDate: selectedDateTimeRange?.endDate?.toISOString(),
          fuelType: (selectedFuelType as string) || "U91",
        });
      return priceTrendsResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching data. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  async function FetchPriceTrendsWithinRadius() {
    try {
      const priceTrendsResponse =
        await priceTrendsApi.apiTrendsDailyAveragesWithinRadiusGet({
          startDate: selectedDateTimeRange?.startDate?.toISOString(),
          endDate: selectedDateTimeRange?.endDate?.toISOString(),
          fuelType: (selectedFuelType as string) || "U91",
          latitude: selectedPosition?.lat,
          longitude: selectedPosition?.long,
          radiusInMetres: selectedRadius * 1000,
        });
      return priceTrendsResponse.data;
    } catch (error) {
      if (axios.isAxiosError(error) && error.response?.status === 404) {
        notifications.show({
          title: "No price trends found",
          message:
            "No price trend data could be found with the selected filters. Please try a broader search.",
          color: "yellow",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      } else {
        notifications.show({
          title: `${error}`,
          message:
            "There was an error while fetching data. Please refresh the page and try again. If the issue persists, please report it to us!",
          color: "red",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      }
    }
  }

  const handleMarkerPlaced = (coordinate: Coordinate) => {
    setSelectedPosition(coordinate);
  };

  const fuelTypesQuery = useQuery({
    queryKey: ["chartsPageFuelTypes"],
    queryFn: FetchFuelTypes,
  });
  const priceTrendsQueryAllStations = useQuery({
    queryKey: ["chartsPagePriceTrendsAllStations"],
    queryFn: FetchPriceTrendsAllStations,
  });
  const priceTrendsQueryWithinRadius = useQuery({
    queryKey: ["chartsPagePriceTrendsWithinRadius"],
    queryFn: FetchPriceTrendsWithinRadius,
  });

  const allStationsChartData = useMemo(() => {
    return priceTrendsQueryAllStations?.data?.dailyAverages?.map((dpa) => {
      return {
        date: dpa.date as string,
        averagePriceValue: dpa.averagePriceValue as number,
      };
    });
  }, [priceTrendsQueryAllStations.data]);

  const withinRadiusChartData = useMemo(() => {
    return priceTrendsQueryWithinRadius?.data?.dailyAverages?.map((dpa) => {
      return {
        date: dpa.date as string,
        averagePriceValue: dpa.averagePriceValue as number,
      };
    });
  }, [priceTrendsQueryWithinRadius.data]);

  const isLoading =
    fuelTypesQuery.isFetching ||
    priceTrendsQueryAllStations.isFetching ||
    priceTrendsQueryWithinRadius.isFetching;

  return (
    <Container size={"95vw"}>
      <Title order={1}>View historic fuel price trends.</Title>
      <Grid gutter="md">
        <Grid.Col span={8}>
          <Stack gap="xl">
            <Group align="end" gap="xs" grow>
              <Select
                label="Fuel Type"
                description="Select a fuel type below"
                placeholder="Select a fuel type"
                data={fuelTypesQuery.data?.map((ft) => ({
                  label: ft.name as string,
                  value: ft.code as string,
                }))}
                value={selectedFuelType}
                onChange={(value) => setSelectedFuelType(value as string)}
                w={220}
              />
              <DateRangePicker
                defaultTimePeriod={TimePeriod.Last30Days}
                onChange={setSelectedDateTimeRange}
              />
              <NumberInput
                label="Radius to search within"
                description="Enter a radius in kilometres"
                placeholder="Enter a number"
                value={selectedRadius}
                onChange={(value) => setSelectedRadius(value as number)}
                suffix=" km"
                w={160}
              />
              <Button
                onClick={() => {
                  priceTrendsQueryAllStations.refetch();
                  priceTrendsQueryWithinRadius.refetch();
                }}
                style={{ minWidth: "100px" }}
                loading={isLoading}
              >
                Apply
              </Button>
            </Group>
            <CustomMap
              centre={selectedPosition}
              height={"65vh"}
              scrollWheelZoom
              circle={{
                centre: selectedPosition as Coordinate,
                radius: selectedRadius * 1000,
              }}
              clickToPlaceMarker
              markerPosition={selectedPosition || undefined}
              onMarkerPlaced={handleMarkerPlaced}
            />
          </Stack>
        </Grid.Col>
        <Grid.Col span={4}>
          <Stack gap="xl">
            <Skeleton visible={isLoading}>
              <Title order={3} pb={20}>
                Daily averages for all stations.
              </Title>
              <LineChart
                h={"34vh"}
                data={
                  allStationsChartData?.map((data) => ({
                    ...data,
                    formattedDate: DateToDayMonthYearAbbrev(
                      new Date(data.date)
                    ),
                  })) || []
                }
                series={[
                  {
                    name: "averagePriceValue",
                    label: "Average Price per Litre",
                  },
                ]}
                dataKey="formattedDate"
                type="gradient"
                gradientStops={[
                  { offset: 0, color: "red" },
                  { offset: 100, color: "green" },
                ]}
                strokeWidth={5}
                curveType="monotone"
                yAxisProps={{
                  domain: [
                    priceTrendsQueryAllStations.data?.statistics
                      ?.dailyAverageMin as number,
                    priceTrendsQueryAllStations.data?.statistics
                      ?.dailyAverageMax as number,
                  ],
                }}
                valueFormatter={(value) => `${value.toFixed(2)}`}
              />
              <Title order={3} py={20}>
                Daily averages for selected radius.
              </Title>
              <LineChart
                h={"34vh"}
                data={
                  withinRadiusChartData?.map((data) => ({
                    ...data,
                    formattedDate: DateToDayMonthYearAbbrev(
                      new Date(data.date)
                    ),
                  })) || []
                }
                series={[
                  {
                    name: "averagePriceValue",
                    label: "Average Price per Litre",
                  },
                ]}
                dataKey="formattedDate"
                type="gradient"
                gradientStops={[
                  { offset: 0, color: "red" },
                  { offset: 100, color: "green" },
                ]}
                strokeWidth={5}
                curveType="monotone"
                yAxisProps={{
                  domain: [
                    priceTrendsQueryWithinRadius.data?.statistics
                      ?.dailyAverageMin as number,
                    priceTrendsQueryWithinRadius.data?.statistics
                      ?.dailyAverageMax as number,
                  ],
                }}
                valueFormatter={(value) => `${value.toFixed(2)}`}
              />
            </Skeleton>
          </Stack>
        </Grid.Col>
      </Grid>
    </Container>
  );
}
