"use client";

import { Checkbox, Pagination, Skeleton, Table } from "@mantine/core";
import { useEffect, useState } from "react";
import { FaAnglesDown, FaAnglesUp } from "react-icons/fa6";
import { ColumnType, GetNestedValue, Path } from "../../../utils/tableUtils";

type CustomTableProps<T> = {
  data: T[];
  columns: ColumnType<T>[];
  sortableColumns?: string[];
  itemsPerPage: number;
  striped?: boolean;
  highlightOnHover?: boolean;
  selectable?: boolean;
  onRowClicked?: (selectedItems: T) => void;
  onSelectionsChange?: (selectedItems: T[]) => void;
  loading?: boolean;
  pagination?: {
    page: number;
    onPageChange: (page: number) => void;
    total: number;
  };
};

export function CustomTable<T>(props: CustomTableProps<T>) {
  const [sortBy, setSortBy] = useState<Path<T> | null>(null);
  const [sortOrder, setSortOrder] = useState<"asc" | "desc" | null>(null);
  const [selectedRows, setSelectedRows] = useState<Set<number>>(new Set());
  const [sortedData, setSortedData] = useState<T[]>(props.data);

  const handleSort = (accessor: Path<T>) => {
    if (sortBy === accessor) {
      setSortOrder(
        sortOrder === "asc" ? "desc" : sortOrder === "desc" ? null : "asc"
      );
      if (sortOrder === "desc") {
        setSortBy(null);
      }
    } else {
      setSortBy(accessor);
      setSortOrder("asc");
    }
  };

  useEffect(() => {
    if (sortBy && sortOrder) {
      const sorted = [...props.data].sort((a, b) => {
        const aValue = GetNestedValue(a, sortBy) || "";
        const bValue = GetNestedValue(b, sortBy) || "";
        if (aValue < bValue) return sortOrder === "asc" ? -1 : 1;
        if (aValue > bValue) return sortOrder === "asc" ? 1 : -1;
        return 0;
      });
      setSortedData(sorted);
    } else {
      setSortedData(props.data);
    }
  }, [props.data, sortBy, sortOrder]);

  const handleSelectRow = (index: number) => {
    const rowIndex =
      (props.pagination?.page ? props.pagination.page - 1 : 0) *
        props.itemsPerPage +
      index;
    const newSelectedRows = new Set(selectedRows);
    if (newSelectedRows.has(rowIndex)) {
      newSelectedRows.delete(rowIndex);
    } else {
      newSelectedRows.add(rowIndex);
    }
    setSelectedRows(newSelectedRows);
  };

  useEffect(() => {
    if (props.onSelectionsChange) {
      const selectedItems = Array.from(selectedRows).map(
        (rowIndex) => props.data[rowIndex]
      );
      props.onSelectionsChange(selectedItems);
    }
  }, [selectedRows]);

  return (
    <Skeleton animate visible={props.loading || false}>
      <div style={{ overflowX: "auto" }}>
        <Table
          striped={props.striped}
          highlightOnHover={props.highlightOnHover}
          style={{ tableLayout: "fixed", width: "100%" }}
        >
          <Table.Thead>
            <Table.Tr>
              {props.selectable && <Table.Th style={{ width: "2%" }} />}
              {props.columns.map((column) => (
                <Table.Th
                  key={column.accessor}
                  onClick={() =>
                    props.sortableColumns?.includes(column.accessor) &&
                    handleSort(column.accessor)
                  }
                  style={{
                    cursor: props.sortableColumns?.includes(column.accessor)
                      ? "pointer"
                      : "default",
                    textAlign: "left",
                    overflow: "hidden",
                    textOverflow: "ellipsis",
                    whiteSpace: "nowrap",
                    padding: "8px 16px",
                    width: column.width || "auto",
                  }}
                >
                  {column.label}{" "}
                  {sortBy === column.accessor &&
                    (sortOrder ? (
                      sortOrder === "asc" ? (
                        <FaAnglesUp />
                      ) : (
                        <FaAnglesDown />
                      )
                    ) : (
                      ""
                    ))}
                </Table.Th>
              ))}
            </Table.Tr>
          </Table.Thead>
          <Table.Tbody>
            {sortedData?.map((item, index) => {
              //don't render any rows beyond the limit defined in the per-page value
              if (index > props.itemsPerPage) {
                return;
              }
              const isSelected = selectedRows.has(index);
              return (
                <Table.Tr
                  style={{ cursor: props.onRowClicked ? "pointer" : undefined }}
                  onClick={() => {
                    if (props.onRowClicked) {
                      props.onRowClicked(props.data[index]);
                    }
                  }}
                  key={index}
                >
                  {props.selectable && (
                    <Table.Td pr={16}>
                      <Checkbox
                        checked={isSelected}
                        onChange={() => handleSelectRow(index)}
                        color="lime"
                      />
                    </Table.Td>
                  )}
                  {props.columns.map((column) => (
                    <Table.Td
                      key={column.accessor}
                      style={{
                        overflow: "hidden",
                        textOverflow: "ellipsis",
                        whiteSpace: "nowrap",
                        padding: "8px 16px",
                        width: column.width || "auto",
                      }}
                    >
                      {column.formatter
                        ? column.formatter(item)
                        : String(GetNestedValue(item, column.accessor))}
                    </Table.Td>
                  ))}
                </Table.Tr>
              );
            })}
          </Table.Tbody>
        </Table>

        {props.pagination && (
          <Pagination
            total={props.pagination.total}
            value={props.pagination.page}
            onChange={props.pagination.onPageChange}
            mt="md"
          />
        )}
      </div>
    </Skeleton>
  );
}
