"use client";

import {
  Container,
  Grid,
  Group,
  InputLabel,
  SegmentedControl,
} from "@mantine/core";
import { DateTimePicker } from "@mantine/dates";
import dayjs from "dayjs";
import utc from "dayjs/plugin/utc";
import { useEffect, useState } from "react";
import { DateTimeRange } from "../../../models/DateTimeRange";
import { TimePeriod, timePeriodOptions } from "../../../models/TimePeriod";
import { getDateRangeForPeriod } from "../../../utils/dateRangePickerUtils";

dayjs.extend(utc);

type DateRangePickerProps = {
  defaultTimePeriod: TimePeriod;
  onChange: (dateTimeRange: DateTimeRange) => void;
};

export default function DateRangePicker(props: DateRangePickerProps) {
  const [startDate, setStartDate] = useState<Date | null>(null);
  const [endDate, setEndDate] = useState<Date | null>(null);
  const [timePeriod, setTimePeriod] = useState<TimePeriod>(
    props.defaultTimePeriod
  );

  //update state based on time period selection
  useEffect(() => {
    const { startDate, endDate } = getDateRangeForPeriod(timePeriod);
    setStartDate(startDate);
    setEndDate(endDate);
  }, [timePeriod]);

  //notify parent component of state changes
  useEffect(() => {
    const utcStartDate = startDate ? dayjs(startDate).utc().toDate() : null;
    const utcEndDate = endDate ? dayjs(endDate).utc().toDate() : null;
    props.onChange({ startDate: utcStartDate, endDate: utcEndDate });
  }, [startDate, endDate, timePeriod]);

  return (
    <Container>
      <Grid mt="sm" align="center">
        <Grid.Col span={12}>
          <InputLabel pb={4}>
            Select a common date range here, or specify an interval below
          </InputLabel>
          <SegmentedControl
            data={timePeriodOptions.map((option) => ({
              label: option.label,
              value: option.value,
            }))}
            value={timePeriod}
            onChange={(value) => setTimePeriod(value as TimePeriod)}
            size="sm"
            fullWidth
          />
        </Grid.Col>
      </Grid>
      <Group grow>
        <DateTimePicker
          label="Start Date"
          value={startDate}
          onChange={(value) => setStartDate(new Date(value as string))}
          clearable
        />
        <DateTimePicker
          label="End Date"
          value={endDate}
          onChange={(value) => setEndDate(new Date(value as string))}
          clearable
        />
      </Group>
    </Container>
  );
}
