import { Badge, Group, Select, Stack, useCombobox } from "@mantine/core";
import { notifications } from "@mantine/notifications";
import axios from "axios";
import { useEffect, useState } from "react";
import { FaAnglesDown } from "react-icons/fa6";
import { FuelType, FuelTypeApi } from "../../../generated";
import { fuelTypeColours } from "../../../utils/fuelTypeUtils";
import { GetDefaultFuelType } from "../../../utils/userPreferencesUtils";

type FuelTypeSelectProps = {
  onDropdownOpen?: () => void;
  onDropdownClose?: () => void;
};

export default function FuelTypeSelect(props: FuelTypeSelectProps) {
  const fuelTypeApi = new FuelTypeApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const [selectedFuelType, setSelectedFuelType] = useState<string | null>(
    GetDefaultFuelType()
  );
  const [fuelTypes, setFuelTypes] = useState<FuelType[]>([]);
  const [seeMoreClicked, setSeeMoreClicked] = useState(false);
  const combobox = useCombobox();

  async function FetchFuelTypes(fetchAll: boolean) {
    try {
      const fuelTypeResponse = fetchAll
        ? await fuelTypeApi.apiFuelTypesAllGet()
        : await fuelTypeApi.apiFuelTypesCommonGet();
      setFuelTypes(fuelTypeResponse.data);
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching the fuel type data. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  const renderFuelTypeOption = (value: string, label: string) => {
    if (value === "see-more" && seeMoreClicked) {
      return;
    }

    if (value === "see-more") {
      return (
        <div
          onClick={() => {
            setSeeMoreClicked(true);
            FetchFuelTypes(true);
          }}
          style={{
            display: "flex",
            alignItems: "center",
            justifyContent: "center",
            width: "100%",
            cursor: "pointer",
          }}
        >
          <FaAnglesDown />
          <span style={{ flex: 1, textAlign: "center" }}>See More</span>
          <FaAnglesDown />
        </div>
      );
    }

    return (
      <Group gap="xs">
        <Badge color={fuelTypeColours[value] || "gray"}>{label}</Badge>
      </Group>
    );
  };

  const dataWithSeeMore = [
    ...fuelTypes.map((ft) => ({
      value: ft.code as string,
      label: ft.name as string,
    })),
    { value: "see-more", label: "See More" },
  ];

  useEffect(() => {
    if (!fuelTypes.length) {
      FetchFuelTypes(false);
    }
  }, []);

  return (
    <Stack>
      <Select
        comboboxProps={{ store: combobox }}
        px={12}
        placeholder="Default fuel type"
        data={dataWithSeeMore}
        value={selectedFuelType}
        onChange={(value) => {
          if (value === "see-more") return;
          setSelectedFuelType(value as string);
          localStorage.setItem("defaultFuelType", value as string);
        }}
        searchable
        renderOption={({ option }) =>
          renderFuelTypeOption(option.value, option.label)
        }
        leftSectionWidth={64}
        leftSection={
          <Badge color={fuelTypeColours[selectedFuelType as string]}>
            {selectedFuelType}
          </Badge>
        }
        onClick={() => combobox.openDropdown()}
        onDropdownOpen={props.onDropdownOpen}
        onDropdownClose={props.onDropdownClose}
      />
    </Stack>
  );
}
