import { Button, Container, Group, Popover, Text } from "@mantine/core";
import { useDisclosure } from "@mantine/hooks";
import Link from "next/link";
import { usePathname, useRouter } from "next/navigation";
import { useEffect, useState } from "react";
import { FaLocationDot } from "react-icons/fa6";
import {
  GetDefaultFuelType,
  GetHomeLocation,
} from "../../../../utils/userPreferencesUtils";
import FuelTypeSelect from "../../fuel-type-select/FuelTypeSelect";
import LocationSelectModal from "../../map/LocationSelectModal";
import ColourSchemeToggle from "./colour-scheme/ColourSchemeToggle";
import classes from "./Header.module.css";

const links = [
  { link: "/", label: "Home" },
  { link: "/map", label: "Map" },
  { link: "/charts", label: "Charts" },
  { link: "/stations", label: "Stations" },
  { link: "/faq", label: "FAQ" },
  { link: "/developers", label: "Developers" },
];

export function Header() {
  const pathName = usePathname();
  const [active, setActive] = useState(pathName);
  const router = useRouter();
  const [locationSelectOpened, locationSelectDisclosureHandler] =
    useDisclosure(false);
  const [hydrated, setHydrated] = useState(false);
  const [homePromptOpen, setHomePromptOpen] = useState(
    GetHomeLocation() == undefined
  );
  const [defaultFuelTypePromptOpen, setDefaultFuelTypePromptOpen] = useState(
    GetDefaultFuelType() == undefined
  );

  const handleSetHomeLocationClicked = () => {
    setHomePromptOpen(false);
    setDefaultFuelTypePromptOpen(false);
    locationSelectDisclosureHandler.open();
  };

  const handleLocationSelectModalClosed = () => {
    locationSelectDisclosureHandler.close();
    setHomePromptOpen(GetHomeLocation() == undefined);
    setDefaultFuelTypePromptOpen(GetDefaultFuelType() == undefined);
  };

  useEffect(() => {
    setHydrated(true);
  }, []);

  return (
    <header className={classes.header}>
      <LocationSelectModal
        opened={locationSelectOpened}
        onClose={handleLocationSelectModalClosed}
        onSelect={(c) => {
          localStorage.setItem("homeLocation", JSON.stringify(c));
        }}
        initialPosition={GetHomeLocation()}
      />
      <Container size="lg" className={classes.inner}>
        <Group className={classes.navLinks} gap={5} visibleFrom="sm">
          {links.map((link) => (
            <Link
              key={link.label}
              href={link.link}
              className={classes.link}
              data-active={active === link.link || undefined}
              onClick={() => {
                setActive(link.link);
                router.push(link.link);
              }}
            >
              {link.label}
            </Link>
          ))}
        </Group>
        <Group>
          <Popover opened={homePromptOpen} withOverlay width="target" withArrow>
            <Popover.Target>
              <Button
                leftSection={<FaLocationDot />}
                onClick={handleSetHomeLocationClicked}
                style={
                  homePromptOpen ? { zIndex: Number.MAX_SAFE_INTEGER } : {}
                }
              >
                Set Home Location
              </Button>
            </Popover.Target>
            <Popover.Dropdown>
              <Text size="sm">This is privately stored on your browser.</Text>
            </Popover.Dropdown>
          </Popover>
          {hydrated && (
            <Popover
              opened={defaultFuelTypePromptOpen}
              withOverlay
              width="target"
              withArrow
            >
              <Popover.Target>
                <div
                  style={
                    defaultFuelTypePromptOpen
                      ? { zIndex: Number.MAX_SAFE_INTEGER }
                      : {}
                  }
                >
                  <FuelTypeSelect
                    onDropdownOpen={() => setDefaultFuelTypePromptOpen(false)}
                    onDropdownClose={() =>
                      setDefaultFuelTypePromptOpen(
                        GetDefaultFuelType() == undefined
                      )
                    }
                  />
                </div>
              </Popover.Target>
              <Popover.Dropdown>
                <Text size="sm">
                  This will be the default fuel type on all pages. You can
                  change this at any time.
                </Text>
              </Popover.Dropdown>
            </Popover>
          )}
        </Group>
        <ColourSchemeToggle />
      </Container>
    </header>
  );
}
