"use client";

import { Button, Loader, Modal, Select } from "@mantine/core";
import { useDebouncedCallback } from "@mantine/hooks";
import { notifications } from "@mantine/notifications";
import { useQuery } from "@tanstack/react-query";
import axios from "axios";
import "leaflet/dist/leaflet.css";
import dynamic from "next/dynamic";
import { useState } from "react";
import { SuburbBoundary, SuburbBoundaryApi } from "../../../generated";
import { Coordinate } from "../../../models/Coordinate";
const CustomMap = dynamic(() => import("./CustomMap"), {
  ssr: false,
});

type LocationSelectModalProps = {
  opened: boolean;
  onClose: () => void;
  onSelect: (coordinates: Coordinate) => void;
  initialPosition?: Coordinate;
};

export default function LocationSelectModal(props: LocationSelectModalProps) {
  const suburbBoundaryApi = new SuburbBoundaryApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );

  const [selectedPosition, setSelectedPosition] = useState<{
    lat: number;
    long: number;
  } | null>(props.initialPosition || null);
  const [suburbSearch, setSuburbSearch] = useState("");
  const [selectedSuburbBoundary, setSelectedSuburbBoundary] =
    useState<SuburbBoundary>();

  const FetchSuburbs = async () => {
    if (suburbSearch == "") {
      return;
    }
    try {
      const suburbResponse =
        await suburbBoundaryApi.apiSuburbBoundariesSuburbsByNameGet({
          name: suburbSearch,
        });
      return suburbResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching the suburb data. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  };

  const suburbsQuery = useQuery({
    queryKey: ["suburbs"],
    queryFn: FetchSuburbs,
  });

  async function FetchSuburbBoundary(id: string | null) {
    if (!id) {
      return;
    }
    try {
      const suburbResponse = await suburbBoundaryApi.apiSuburbBoundariesByIdGet(
        { id: id }
      );
      setSelectedSuburbBoundary(suburbResponse.data);
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching the suburb boundary data. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  //debounced callback to delay the FetchSuburbs call until after typing
  const debouncedFetchSuburbs = useDebouncedCallback(() => {
    suburbsQuery.refetch();
  }, 1000);

  const handleSearchChange = (value: string) => {
    setSuburbSearch(value);
    debouncedFetchSuburbs();
  };

  const handleSelectedSuburbChange = (value: string | null) => {
    FetchSuburbBoundary(value);
  };

  const handleMarkerPlaced = (coordinates: { lat: number; long: number }) => {
    setSelectedPosition(coordinates);
  };

  const handleConfirm = () => {
    if (selectedPosition) {
      props.onSelect(selectedPosition);
      props.onClose();
    }
  };

  return (
    <Modal
      opened={props.opened}
      onClose={props.onClose}
      title="Click to set your home location"
      size="xl"
      centered
    >
      <Select
        pb={12}
        label="Search for a suburb"
        description="Start typing the name of a suburb, then click to select"
        placeholder="Enter a suburb name"
        data={suburbsQuery.data?.map((s) => {
          return { value: s.id as string, label: s.name as string };
        })}
        searchable
        searchValue={suburbSearch}
        onSearchChange={handleSearchChange}
        onChange={handleSelectedSuburbChange}
        rightSection={suburbsQuery.isFetching && <Loader size="sm" />}
      />
      <CustomMap
        height={"65vh"}
        scrollWheelZoom
        clickToPlaceMarker
        markerPosition={selectedPosition || undefined}
        onMarkerPlaced={handleMarkerPlaced}
        centre={props.initialPosition}
        zoom={props.initialPosition ? 13 : 10}
        suburbBoundary={selectedSuburbBoundary}
      />
      <div
        style={{
          marginTop: "1rem",
          display: "flex",
          justifyContent: "flex-end",
          gap: "1rem",
        }}
      >
        <Button variant="default" onClick={props.onClose}>
          Cancel
        </Button>
        <Button disabled={!selectedPosition} onClick={handleConfirm}>
          Confirm
        </Button>
      </div>
    </Modal>
  );
}
