import { Badge, Group, Text, Timeline } from "@mantine/core";
import dayjs from "dayjs";
import { useState } from "react";
import {
  FaCheck,
  FaClock,
  FaLocationDot,
  FaRoute,
  FaStopwatch,
} from "react-icons/fa6";
import {
  TbArrowMergeAltLeft,
  TbArrowRight,
  TbArrowRoundaboutRight,
  TbCornerUpLeft,
  TbCornerUpRight,
  TbFlag,
  TbFlagCheck,
  TbRoad,
} from "react-icons/tb";
import type {
  OSRMManeuver,
  OSRMRouteResponse,
  OSRMStep,
} from "../../../utils/osrmUtils";

type RouteDisplayProps = {
  osrmRoute?: OSRMRouteResponse;
};

function FormatDistance(distance: number) {
  return distance < 1000
    ? `${distance.toFixed(0)} m`
    : `${(distance / 1000).toFixed(1)} km`;
}

function FormatDuration(duration: number) {
  return duration < 60
    ? `${duration.toFixed(0)} sec`
    : `${(duration / 60).toFixed(0)} min`;
}

function getIconForManeuver(maneuver: OSRMManeuver) {
  switch (maneuver.type) {
    case "depart":
      return <TbFlag />;
    case "arrive":
      return <TbFlagCheck />;
    case "turn":
      if (maneuver.modifier === "left") {
        return <TbCornerUpLeft />;
      } else if (maneuver.modifier === "right") {
        return <TbCornerUpRight />;
      } else {
        return <TbArrowRight />;
      }
    case "merge":
      return <TbArrowMergeAltLeft />;
    case "roundabout":
      return <TbArrowRoundaboutRight />;
    default:
      return <TbRoad />;
  }
}

function formatManeuverText(maneuver: OSRMManeuver): string {
  switch (maneuver.type) {
    case "depart":
      return "Depart";
    case "arrive":
      return "Arrive";
    case "turn":
      return maneuver.modifier ? `Turn ${maneuver.modifier}` : "Turn";
    case "merge":
      return maneuver.modifier ? `Merge ${maneuver.modifier}` : "Merge";
    case "roundabout":
      return maneuver.modifier
        ? `Roundabout ${maneuver.modifier}`
        : "Roundabout";
    default:
      return maneuver.type.charAt(0).toUpperCase() + maneuver.type.slice(1);
  }
}

export default function RouteDisplay(props: RouteDisplayProps) {
  const [active, setActive] = useState(0);

  if (
    !props.osrmRoute?.routes ||
    props.osrmRoute.routes.length === 0 ||
    !props.osrmRoute.waypoints ||
    props.osrmRoute.waypoints.length < 2
  ) {
    return <Text>Invalid route data</Text>;
  }

  const mainRoute = props.osrmRoute.routes[0];
  const steps: OSRMStep[] = mainRoute.legs.flatMap((leg) => leg.steps);
  const departureName = props.osrmRoute.waypoints[0].name || "Start";
  const arrivalName =
    props.osrmRoute.waypoints[props.osrmRoute.waypoints.length - 1].name ||
    "Destination";

  return (
    <>
      <Group>
        <Badge size="xl" leftSection={<FaRoute />}>
          {FormatDistance(props.osrmRoute.routes[0].distance)}
        </Badge>
        <Badge size="xl" leftSection={<FaStopwatch />}>
          {FormatDuration(props.osrmRoute.routes[0].duration)}
        </Badge>
        <Badge size="xl" leftSection={<FaClock />}>
          Arrive at{" "}
          {dayjs()
            .add(props.osrmRoute.routes[0].duration, "seconds")
            .format("HH:mm")}
        </Badge>
      </Group>
      <Timeline active={active} bulletSize={36} lineWidth={2} pt={24}>
        <Timeline.Item
          bullet={<FaLocationDot />}
          title={departureName}
          onClick={() => setActive(0)}
        >
          <Text c="dimmed" size="sm">
            Departure
          </Text>
        </Timeline.Item>

        {steps.map((step, index) => (
          <Timeline.Item
            key={index}
            bullet={getIconForManeuver(step.maneuver)}
            title={`Step ${index + 1}: ${formatManeuverText(step.maneuver)}${
              step.name ? ` onto ${step.name}` : ""
            }`}
            onClick={() => setActive(index + 1)}
          >
            <Text c="dimmed" size="sm">
              Distance: {FormatDistance(step.distance)} &nbsp;|&nbsp; Duration:{" "}
              {FormatDuration(step.duration)}
            </Text>
          </Timeline.Item>
        ))}

        <Timeline.Item
          bullet={<FaCheck />}
          title={arrivalName}
          onClick={() => setActive(steps.length + 1)}
        >
          <Text c="dimmed" size="sm">
            Arrival
          </Text>
        </Timeline.Item>
      </Timeline>
    </>
  );
}
