"use client";

import {
  Button,
  Container,
  Drawer,
  Group,
  NumberInput,
  Select,
  Stack,
  Text,
  Title,
} from "@mantine/core";
import { useDisclosure } from "@mantine/hooks";
import { notifications } from "@mantine/notifications";
import { useQuery } from "@tanstack/react-query";
import axios from "axios";
import { LineString } from "geojson";
import "leaflet/dist/leaflet.css";
import dynamic from "next/dynamic";
import { useEffect, useState } from "react";
import { FaRoute } from "react-icons/fa6";
import {
  BrandApi,
  BrandType,
  FuelType,
  FuelTypeApi,
  StationWithPrices,
  StationWithPricesApi,
} from "../../generated";
import { Coordinate } from "../../models/Coordinate";
import { getRoute, OSRMRouteResponse } from "../../utils/osrmUtils";
import {
  GetDefaultFuelType,
  GetDefaultLocation,
  GetHomeLocation,
} from "../../utils/userPreferencesUtils";
import { CustomMapPoint } from "../components/map/CustomMap";
import RouteDisplay from "../components/map/RouteDisplay";
import StationsWithPricesTable from "../stations/StationsWithPricesTable";
const CustomMap = dynamic(() => import("../components/map/CustomMap"), {
  ssr: false,
});

export default function MapPage() {
  const homeLocation = GetHomeLocation();
  const defaultLocation = GetDefaultLocation();
  const stationWithPricesApi = new StationWithPricesApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const brandApi = new BrandApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const fuelTypeApi = new FuelTypeApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const [selectedPoint, setSelectedPoint] = useState<
    CustomMapPoint | undefined
  >(undefined);

  const [selectedFuelType, setSelectedFuelType] = useState(
    GetDefaultFuelType()
  );
  const [selectedRadius, setSelectedRadius] = useState(10);
  const [selectedPosition, setSelectedPosition] = useState<
    Coordinate | undefined
  >(homeLocation);
  const [selectedBrand, setSelectedBrand] = useState<string>();
  const [selectedStationCoordinates, setSelectedStationCoordinates] =
    useState<Coordinate | null>(null);

  const [directionsDrawerOpened, directionsDrawerHandler] =
    useDisclosure(false);

  async function FetchBrands() {
    try {
      const brandResponse = await brandApi.apiBrandsAllGet();
      return brandResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching brands. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  async function FetchFuelTypes() {
    try {
      const fuelTypeResponse = await fuelTypeApi.apiFuelTypesAllGet();
      return fuelTypeResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching fuel types. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  async function FetchStationsWithPrices() {
    try {
      const stationsWithPricesResponse =
        await stationWithPricesApi.apiStationsWithPricesWithinRadiusGet({
          brand: selectedBrand,
          fuelType: selectedFuelType as string,
          pageNumber: 1,
          pageSize: 100,
          radiusInMetres: selectedRadius * 1000,
          latitude: selectedPosition?.lat || defaultLocation?.lat,
          longitude: selectedPosition?.long || defaultLocation?.long,
        });

      if (stationsWithPricesResponse.data.data?.length === 100) {
        notifications.show({
          title: "Station limit reached",
          message:
            "There are a lot of stations within that radius, showing only the cheapest 100 stations.",
          color: "yellow",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      }
      return stationsWithPricesResponse.data.data;
    } catch (error) {
      if (axios.isAxiosError(error) && error.response?.status === 404) {
        notifications.show({
          title: "No stations found",
          message:
            "No stations could be found with the selected filters. Please try a broader search.",
          color: "yellow",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      } else {
        notifications.show({
          title: `${error}`,
          message:
            "There was an error while fetching data. Please refresh the page and try again. If the issue persists, please report it to us!",
          color: "red",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      }
    }
  }

  async function FetchOSRMRoute() {
    if (selectedStationCoordinates !== null) {
      return await getRoute(
        [
          selectedPosition as Coordinate,
          selectedStationCoordinates as Coordinate,
        ],
        { overview: "full", profile: "driving", steps: true }
      );
    }
  }

  const handleMarkerPlaced = (coordinate: Coordinate) => {
    setSelectedPosition(coordinate);
  };

  const handleClickDirections = (coordinate: Coordinate) => {
    setSelectedStationCoordinates(coordinate);
    directionsDrawerHandler.open();
  };

  const brandsQuery = useQuery({
    queryKey: ["mapPageBrands"],
    queryFn: FetchBrands,
  });
  const fuelTypesQuery = useQuery({
    queryKey: ["mapPageFuelTypes"],
    queryFn: FetchFuelTypes,
  });
  const stationsWithPricesQuery = useQuery({
    queryKey: ["mapPageStationsWithPrices"],
    queryFn: FetchStationsWithPrices,
  });
  const osrmQuery = useQuery({
    queryKey: ["mapPageOsrm"],
    queryFn: FetchOSRMRoute,
  });

  useEffect(() => {
    if (selectedStationCoordinates !== null) {
      osrmQuery.refetch();
    }
  }, [selectedStationCoordinates]);

  const isLoading =
    brandsQuery.isFetching ||
    fuelTypesQuery.isFetching ||
    stationsWithPricesQuery.isFetching;

  return (
    <Container size={"95vw"}>
      <Drawer
        opened={directionsDrawerOpened}
        onClose={directionsDrawerHandler.close}
        withOverlay={false}
        position="right"
        size="lg"
      >
        <RouteDisplay osrmRoute={osrmQuery.data as OSRMRouteResponse} />
      </Drawer>
      <Title order={1} pb={10}>
        View the prices for a location.
      </Title>
      <Group justify="space-between" grow align="flex-start">
        <Stack align="stretch" justify="center" gap="xl">
          <Group align="end" grow>
            <Select
              label="Fuel Type"
              description="Select a fuel type below"
              placeholder="Select a fuel type"
              data={fuelTypesQuery.data?.map((ft: FuelType) => {
                return {
                  label: ft.name as string,
                  value: ft.code as string,
                };
              })}
              value={selectedFuelType}
              onChange={(value) => setSelectedFuelType(value)}
              searchable
            />
            <NumberInput
              label="Radius to search within"
              description="Enter a radius in kilometres"
              placeholder="Enter a number"
              value={selectedRadius}
              onChange={(value) => setSelectedRadius(value as number)}
              suffix=" km"
            />
            <Select
              label="Filter by brand"
              description="Once selected, press the X to clear"
              placeholder="Select a brand"
              data={brandsQuery.data?.map((b: BrandType) => {
                return { label: b.name as string, value: b.name as string };
              })}
              value={selectedBrand}
              onChange={(value) => setSelectedBrand(value as string)}
              searchable
              clearable
            />
            <Button
              onClick={() => stationsWithPricesQuery.refetch()}
              loading={isLoading}
            >
              Apply
            </Button>
          </Group>
          <CustomMap
            selectedPoint={selectedPoint}
            centre={selectedPosition}
            height={"70vh"}
            scrollWheelZoom
            circle={{
              centre: selectedPosition as Coordinate,
              radius: selectedRadius * 1000,
            }}
            clickToPlaceMarker
            markerPosition={selectedPosition || undefined}
            onMarkerPlaced={handleMarkerPlaced}
            points={stationsWithPricesQuery.data?.map(
              (swp: StationWithPrices) => {
                return {
                  lat: swp.station?.coordinates?.latitude,
                  long: swp.station?.coordinates?.longitude,
                  popup: (
                    <Container p={"xs"}>
                      <Title order={3}>{swp.station?.name}</Title>
                      <Title order={1}>
                        {
                          swp.prices?.find(
                            (p) => p.fueltype === selectedFuelType
                          )?.price
                        }
                      </Title>
                      <Text>{swp.station?.address}</Text>
                      <Button
                        fullWidth
                        leftSection={<FaRoute />}
                        onClick={() =>
                          handleClickDirections({
                            lat: swp.station?.coordinates?.latitude as number,
                            long: swp.station?.coordinates?.longitude as number,
                          })
                        }
                        loading={osrmQuery.isFetching}
                      >
                        Directions
                      </Button>
                    </Container>
                  ),
                } as CustomMapPoint;
              }
            )}
            routeGeoJson={{
              geometry: osrmQuery.data?.routes[0].geometry as LineString,
              type: "Feature",
              bbox: osrmQuery.data?.routes[0].geometry.bbox,
              properties: {},
            }}
          />
        </Stack>
        <Stack>
          <Title order={3}>Cheapest stations in radius.</Title>
          <StationsWithPricesTable
            stationsWithPrices={stationsWithPricesQuery.data || []}
            totalPages={1}
            currentPage={1}
            onPageChange={() => {}}
            itemsPerPage={15}
            loading={isLoading}
            onRowClicked={(selection) => {
              if (selection.station?.coordinates) {
                setSelectedPoint({
                  lat: selection.station.coordinates.latitude!,
                  long: selection.station.coordinates.longitude!,
                });
              }
            }}
          />
        </Stack>
      </Group>
    </Container>
  );
}
