import { useMemo } from "react";
import { StationWithPrices } from "../../generated";
import { ColumnType, Path } from "../../utils/tableUtils";
import { CustomTable } from "../components/custom-table/CustomTable";

type StationsWithPricesTableProps = {
  stationsWithPrices: StationWithPrices[];
  totalPages: number;
  currentPage: number;
  onPageChange: (page: number) => void;
  showAddress?: boolean;
  showPagination?: boolean;
  itemsPerPage?: number;
  loading?: boolean;
  selectable?: boolean;
  onRowClicked?: (selectedStation: StationWithPrices) => void;
  onSelectionChange?: (selectedStations: StationWithPrices[]) => void;
};

export default function StationsWithPricesTable(
  props: StationsWithPricesTableProps
) {
  const columns = useMemo<ColumnType<StationWithPrices>[]>(
    () => [
      {
        label: "Name",
        accessor: "station.name" as Path<StationWithPrices>,
      },
      {
        label: "Price",
        accessor: "prices[0].price" as Path<StationWithPrices>,
        width: 90,
      },
      {
        label: "Brand",
        accessor: "station.brand" as Path<StationWithPrices>,
        width: 200,
      },
      {
        label: "Distance (km)",
        accessor: "station.distance" as Path<StationWithPrices>,
        formatter: (value) => {
          const distanceKm = (value.station?.distance as number) / 1000;
          return `${distanceKm.toFixed(1)}`;
        },
        width: 140,
      },
      ...(props.showAddress
        ? [
            {
              label: "Address",
              accessor: "station.address" as Path<StationWithPrices>,
            },
          ]
        : []),
    ],
    [props.showAddress]
  );

  return (
    <CustomTable<StationWithPrices>
      data={props.stationsWithPrices}
      columns={columns}
      // Sorting does not work across pages, so for now, until we have API-side sorting, deactivate this
      // sortableColumns={['station.name', 'prices[0].price', 'station.brand']}
      itemsPerPage={props.itemsPerPage || 25}
      striped
      highlightOnHover
      pagination={
        props.showPagination
          ? {
              total: props.totalPages,
              page: props.currentPage,
              onPageChange: props.onPageChange,
            }
          : undefined
      }
      loading={props.loading}
      selectable={props.selectable}
      onSelectionsChange={props?.onSelectionChange}
      onRowClicked={props.onRowClicked}
    />
  );
}
