"use client";

import { LineChart } from "@mantine/charts";
import {
  Button,
  Container,
  Grid,
  Group,
  NumberInput,
  Select,
  Stack,
  TextInput,
  Title,
} from "@mantine/core";
import { usePagination } from "@mantine/hooks";
import { notifications } from "@mantine/notifications";
import { useQuery } from "@tanstack/react-query";
import axios from "axios";
import { useEffect, useMemo, useState } from "react";
import {
  BrandApi,
  FuelTypeApi,
  PriceTrendsApi,
  StationWithPrices,
  StationWithPricesApi,
} from "../../generated";
import { DateTimeRange } from "../../models/DateTimeRange";
import { TimePeriod } from "../../models/TimePeriod";
import { getDateRangeForPeriod } from "../../utils/dateRangePickerUtils";
import { DateToDayMonthYearAbbrev } from "../../utils/dateStringUtils";
import {
  GetDefaultFuelType,
  GetDefaultLocation,
  GetHomeLocation,
} from "../../utils/userPreferencesUtils";
import DateRangePicker from "../components/dates/DateRangePicker";
import StationsWithPricesTable from "./StationsWithPricesTable";

export default function StationsPage() {
  const stationWithPricesApi = new StationWithPricesApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const brandApi = new BrandApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const fuelTypeApi = new FuelTypeApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );
  const priceTrendsApi = new PriceTrendsApi(
    undefined,
    process.env.NEXT_PUBLIC_API_BASE_URL,
    axios
  );

  const [selectedBrand, setSelectedBrand] = useState<string>();
  const [selectedFuelType, setSelectedFuelType] = useState(
    GetDefaultFuelType()
  );
  const [selectedRadius, setSelectedRadius] = useState(10);
  const [searchTerm, setSearchTerm] = useState<string>();
  const [totalPages, setTotalPages] = useState(1);
  const [selectedStations, setSelectedStations] =
    useState<StationWithPrices[]>();
  const [selectedDateTimeRange, setSelectedDateTimeRange] =
    useState<DateTimeRange>(getDateRangeForPeriod(TimePeriod.Last30Days));
  const pagination = usePagination({
    total: totalPages,
    initialPage: 1,
  });

  async function FetchBrands() {
    try {
      const brandResponse = await brandApi.apiBrandsAllGet();
      return brandResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching brands. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  async function FetchFuelTypes() {
    try {
      const fuelTypeResponse = await fuelTypeApi.apiFuelTypesAllGet();
      return fuelTypeResponse.data;
    } catch (error) {
      notifications.show({
        title: `${error}`,
        message:
          "There was an error while fetching fuel types. Please refresh the page and try again. If the issue persists, please report it to us!",
        color: "red",
        position: "top-right",
        withCloseButton: true,
        autoClose: 8000,
      });
    }
  }

  async function FetchPriceTrends() {
    if (!selectedStations || !selectedStations.length) {
      return;
    }
    try {
      const priceTrendsResponse =
        await priceTrendsApi.apiTrendsDailyAveragesStationCodesGet({
          startDate: selectedDateTimeRange?.startDate?.toISOString(),
          endDate: selectedDateTimeRange?.endDate?.toISOString(),
          fuelType: (selectedFuelType as string) || "U91",
          stationCodes: selectedStations?.map(
            (s) => s.station?.code
          ) as number[],
        });
      return priceTrendsResponse.data;
    } catch (error) {
      if (axios.isAxiosError(error) && error.response?.status === 404) {
        notifications.show({
          title: "No price trends found",
          message:
            "No price trend data could be found with the selected filters. Please try a broader search.",
          color: "yellow",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      } else {
        notifications.show({
          title: `${error}`,
          message:
            "There was an error while fetching data. Please refresh the page and try again. If the issue persists, please report it to us!",
          color: "red",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      }
    }
  }

  async function FetchStationsWithPrices() {
    try {
      const home = GetHomeLocation();
      const stationsWithPricesResponse =
        await stationWithPricesApi.apiStationsWithPricesWithinRadiusGet({
          brand: selectedBrand,
          fuelType: selectedFuelType as string,
          stationName: searchTerm,
          pageNumber: pagination.active,
          pageSize: 25,
          radiusInMetres: selectedRadius * 1000,
          latitude: home?.lat || GetDefaultLocation()?.lat,
          longitude: home?.long || GetDefaultLocation()?.long,
        });
      setTotalPages(stationsWithPricesResponse.data.totalPages || 1);
      return stationsWithPricesResponse.data.data;
    } catch (error) {
      if (axios.isAxiosError(error) && error.response?.status === 404) {
        notifications.show({
          title: "No stations found",
          message:
            "No stations could be found with the selected filters. Please try a broader search.",
          color: "yellow",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      } else {
        notifications.show({
          title: `${error}`,
          message:
            "There was an error while fetching data. Please refresh the page and try again. If the issue persists, please report it to us!",
          color: "red",
          position: "top-right",
          withCloseButton: true,
          autoClose: 8000,
        });
      }
    }
  }

  const brandsQuery = useQuery({
    queryKey: ["stationsPageBrands"],
    queryFn: FetchBrands,
  });
  const fuelTypesQuery = useQuery({
    queryKey: ["stationsPageFuelTypes"],
    queryFn: FetchFuelTypes,
  });
  const stationsWithPricesQuery = useQuery({
    queryKey: ["stationsPageStationsWithPrices"],
    queryFn: FetchStationsWithPrices,
  });
  const priceTrendsQuery = useQuery({
    queryKey: ["stationsPagePriceTrends"],
    queryFn: FetchPriceTrends,
  });

  const chartData = useMemo(() => {
    return priceTrendsQuery?.data?.dailyAverages?.map((dpa) => ({
      date: dpa.date as string,
      averagePriceValue: dpa.averagePriceValue as number,
    }));
  }, [priceTrendsQuery.data]);

  const isLoading =
    brandsQuery.isFetching ||
    fuelTypesQuery.isFetching ||
    stationsWithPricesQuery.isFetching ||
    priceTrendsQuery.isFetching;

  const handleApply = () => {
    stationsWithPricesQuery.refetch();
    priceTrendsQuery.refetch();
  };

  useEffect(() => {
    stationsWithPricesQuery.refetch();
  }, [pagination.active]);

  return (
    <Container size="95vw">
      <Title order={1} mb="md">
        View and search for stations.
      </Title>
      <Grid gutter="md">
        <Grid.Col span={8}>
          <Stack gap="md">
            <Group align="end" grow>
              <Select
                label="Fuel Type"
                description="Select a fuel type below"
                placeholder="Select a fuel type"
                data={fuelTypesQuery.data?.map((ft) => ({
                  label: ft.name as string,
                  value: ft.code as string,
                }))}
                value={selectedFuelType}
                onChange={(value) => setSelectedFuelType(value as string)}
                searchable
                style={{ flex: 1 }}
              />
              <NumberInput
                label="Radius to search within"
                description="Enter a radius in kilometres from your home location"
                placeholder="Enter a number"
                value={selectedRadius}
                onChange={(value) => setSelectedRadius(value as number)}
                suffix=" km"
                style={{ flex: 1 }}
              />
            </Group>
            <Group align="end" grow>
              <Select
                label="Filter by brand"
                description="Once selected, press the X to clear"
                placeholder="Select a brand"
                data={brandsQuery.data?.map((b) => ({
                  label: b.name as string,
                  value: b.name as string,
                }))}
                value={selectedBrand}
                onChange={(value) => setSelectedBrand(value as string)}
                searchable
                clearable
                style={{ flex: 1 }}
              />
              <TextInput
                label="Filter by name"
                description="Enter the name of the station to search for"
                placeholder="Enter a name"
                value={searchTerm}
                onChange={(event) => setSearchTerm(event.currentTarget.value)}
                style={{ flex: 1 }}
              />
              <Button loading={isLoading} onClick={handleApply}>
                Apply
              </Button>
            </Group>
            <StationsWithPricesTable
              stationsWithPrices={
                stationsWithPricesQuery.data as StationWithPrices[]
              }
              totalPages={totalPages}
              currentPage={pagination.active}
              onPageChange={pagination.setPage}
              showAddress
              showPagination
              loading={isLoading}
              selectable
              onSelectionChange={setSelectedStations}
            />
          </Stack>
        </Grid.Col>
        <Grid.Col span={4}>
          <Title order={2} pl={60} pb={20}>
            Price history for selected stations.
          </Title>
          <Stack gap="md">
            <DateRangePicker
              defaultTimePeriod={TimePeriod.Last30Days}
              onChange={setSelectedDateTimeRange}
            />
            <LineChart
              h="60vh"
              data={
                chartData?.map((data) => ({
                  ...data,
                  formattedDate: DateToDayMonthYearAbbrev(new Date(data.date)),
                })) || []
              }
              series={[
                {
                  name: "averagePriceValue",
                  label: "Average Price per Litre",
                },
              ]}
              dataKey="formattedDate"
              type="gradient"
              gradientStops={[
                { offset: 0, color: "red" },
                { offset: 100, color: "green" },
              ]}
              strokeWidth={5}
              curveType="monotone"
              yAxisProps={{
                domain: [
                  priceTrendsQuery.data?.statistics?.dailyAverageMin as number,
                  priceTrendsQuery.data?.statistics?.dailyAverageMax as number,
                ],
              }}
              valueFormatter={(value) => `${value.toFixed(2)}`}
            />
          </Stack>
        </Grid.Col>
      </Grid>
    </Container>
  );
}
