import L from "leaflet";
import shadow from "leaflet/dist/images/marker-shadow.png";
import { useCallback, useEffect } from "react";
import { useMap } from "react-leaflet";
import markerDefault from "../public/FA5_Map_Marker-default.svg";
import markerLime from "../public/FA5_Map_Marker-lime.svg";
import markerRed from "../public/FA5_Map_Marker-red.svg";
import markerWhite from "../public/FA5_Map_Marker-white.svg";

//utility function to apply dark mode filter to map tiles
export function applyTileFilter(colorScheme: string) {
  const leafletTiles = document.querySelectorAll(".leaflet-tile");
  const mapContainer = document.querySelector(".leaflet-container");

  if (colorScheme === "dark") {
    leafletTiles.forEach((tile) => {
      (tile as HTMLElement).style.filter =
        "brightness(0.6) invert(1) contrast(3) hue-rotate(200deg) saturate(0.3) brightness(0.7)";
    });
    if (mapContainer) {
      (mapContainer as HTMLElement).style.background = "#303030";
    }
  } else {
    leafletTiles.forEach((tile) => {
      (tile as HTMLElement).style.filter = "none";
    });
    if (mapContainer) {
      (mapContainer as HTMLElement).style.background = "";
    }
  }
}

//custom hook to manage Leaflet map tile filtering based on colour scheme
export function useTileFilter(colorScheme: string) {
  const map = useMap();

  const handleTileLoad = useCallback(() => {
    applyTileFilter(colorScheme);
  }, [colorScheme]);

  useEffect(() => {
    //apply the filter whenever a tileload event is seen
    map.on("tileload", handleTileLoad);

    //gross hack to directly modify the css class
    const leafletTileLayer = document.querySelector(".leaflet-tile-pane");
    if (leafletTileLayer) {
      (leafletTileLayer as HTMLElement).style.filter =
        colorScheme === "dark"
          ? "brightness(0.6) invert(1) contrast(3) hue-rotate(200deg) saturate(0.3) brightness(0.7)"
          : "none";
    }

    //cleanup event listener when component is unmounted or colorScheme changes
    return () => {
      map.off("tileload", handleTileLoad);
    };
  }, [map, colorScheme, handleTileLoad]);
}

export const defaultIcon = L.icon({
  iconUrl: markerDefault.src,
  shadowUrl: shadow.src,
  iconSize: [25, 41],
  iconAnchor: [15, 30],
  popupAnchor: [1, -34],
  shadowSize: [41, 41],
});

export const limeIcon = L.icon({
  iconUrl: markerLime.src,
  shadowUrl: shadow.src,
  iconSize: [25, 41],
  iconAnchor: [15, 30],
  popupAnchor: [1, -34],
  shadowSize: [41, 41],
});

export const whiteIcon = L.icon({
  iconUrl: markerWhite.src,
  shadowUrl: shadow.src,
  iconSize: [25, 41],
  iconAnchor: [15, 30],
  popupAnchor: [1, -34],
  shadowSize: [41, 41],
});

export const redIcon = L.icon({
  iconUrl: markerRed.src,
  shadowUrl: shadow.src,
  iconSize: [25, 41],
  iconAnchor: [15, 30],
  popupAnchor: [1, -34],
  shadowSize: [41, 41],
});
