import axios, { AxiosInstance } from "axios";
import { LineString, MultiLineString } from "geojson";
import { Coordinate } from "../models/Coordinate";

export interface OSRMRouteResponse {
  code: string;
  routes: OSRMRoute[];
  waypoints: OSRMWaypoint[];
}

export interface OSRMRoute {
  distance: number;
  duration: number;
  geometry: LineString;
  legs: OSRMLeg[];
  weight_name: string;
  weight: number;
}

export interface OSRMLeg {
  steps: OSRMStep[];
  distance: number;
  duration: number;
  summary: string;
}

export interface OSRMStep {
  distance: number;
  duration: number;
  geometry: LineString | MultiLineString;
  name: string;
  maneuver: OSRMManeuver;
}

export interface OSRMManeuver {
  type: string;
  modifier?: string;
  location: Coordinate;
  bearing_before: number;
  bearing_after: number;
}

export interface OSRMWaypoint {
  name: string;
  location: Coordinate;
  hint: string;
}

export interface OSRMNearestResponse {
  code: string;
  waypoints: OSRMNearestWaypoint[];
}

export interface OSRMNearestWaypoint {
  name: string;
  distance: number;
  location: Coordinate;
  hint: string;
}

export interface OSRMTableResponse {
  code: string;
  distances: number[][];
  durations: number[][];
  sources: OSRMWaypoint[];
  destinations: OSRMWaypoint[];
}

export interface RouteOptions {
  profile?: "driving" | "walking" | "cycling";
  overview?: "full" | "simplified" | "false";
  steps?: boolean;
  alternatives?: boolean;
}

export type OSRMProfile = "driving" | "walking" | "cycling";

//add fallback here if desired
const OSRM_BASE_URL = process.env.NEXT_PUBLIC_OSRM_BASE_URL || "";

function formatCoordinate(coord: Coordinate): string {
  return `${coord.long},${coord.lat}`;
}

function getCoordinatesString(coordinates: Coordinate[]): string {
  return coordinates.map(formatCoordinate).join(";");
}

function createAxiosInstance(baseUrl: string): AxiosInstance {
  return axios.create({
    baseURL: baseUrl,
  });
}

async function axiosGet<T>(
  endpoint: string,
  params: Record<string, any>
): Promise<T> {
  const api: AxiosInstance = createAxiosInstance(OSRM_BASE_URL);
  try {
    const response = await api.get<T>(endpoint, { params });
    return response.data;
  } catch (error: any) {
    throw new Error(`Error fetching ${endpoint}: ${error.message}`);
  }
}

export async function getRoute(
  coordinates: Coordinate[],
  options?: RouteOptions
): Promise<OSRMRouteResponse> {
  if (coordinates.length < 2) {
    throw new Error("At least two coordinates are required to get a route");
  }
  const profile = options?.profile || "driving";
  const coordString = getCoordinatesString(coordinates);

  const params: Record<string, any> = {
    geometries: "geojson",
    overview: options?.overview || "full",
  };

  if (options?.steps) {
    params.steps = "true";
  }
  if (options?.alternatives !== undefined) {
    params.alternatives = options.alternatives ? "true" : "false";
  }

  const endpoint = `/route/v1/${profile}/${coordString}`;
  return axiosGet<OSRMRouteResponse>(endpoint, params);
}

export async function getNearest(
  coordinate: Coordinate,
  profile: OSRMProfile = "driving",
  number: number = 1
): Promise<OSRMNearestResponse> {
  const coordString = formatCoordinate(coordinate);
  const params = { number: number.toString() };
  const endpoint = `/nearest/v1/${profile}/${coordString}`;
  return axiosGet<OSRMNearestResponse>(endpoint, params);
}

export async function getTable(
  coordinates: Coordinate[],
  profile: OSRMProfile = "driving"
): Promise<OSRMTableResponse> {
  if (coordinates.length < 1) {
    throw new Error("At least one coordinate is required to get a table");
  }
  const coordString = getCoordinatesString(coordinates);
  const params = { annotations: "duration,distance" };
  const endpoint = `/table/v1/${profile}/${coordString}`;
  return axiosGet<OSRMTableResponse>(endpoint, params);
}

export async function getMatch(
  coordinates: Coordinate[],
  profile: OSRMProfile = "driving",
  options?: { overview?: "full" | "simplified" | "false"; steps?: boolean }
): Promise<OSRMRouteResponse> {
  if (coordinates.length < 1) {
    throw new Error("At least one coordinate is required to get a match");
  }
  const coordString = getCoordinatesString(coordinates);
  const params: Record<string, any> = {
    geometries: "geojson",
    overview: options?.overview || "full",
  };

  if (options?.steps) {
    params.steps = "true";
  }

  const endpoint = `/match/v1/${profile}/${coordString}`;
  return axiosGet<OSRMRouteResponse>(endpoint, params);
}
